// 
//    Distributed SPH Fluid Simulator (asterix@index.hu)
// =============================================================================
//
// Copyright 2007 David Illes
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 

#ifndef CONTAINER_H
#define CONTAINER_H

#include <vector>

#include "vector.hpp"
#include "particlesystem.hpp"
#include "gpuSPH.hpp"

//-----------------------------------------------------------------------------
// container description class
//=============================================================================
class Container {
//=============================================================================
private:
	ParticleSystem*	partsystem;					// cpu particle system
	GpuSPH 			gpuSPH;						// gpu-accelerated particle system

public:	
	int		id;
	int		rootid;
	
	int 	framenum;							// frame number
	
	std::vector<Container*>	subcontainers;		// subcontainers
	
	bool	show;								// show the container or not
	float	width,height,depth;					// dimensions
	float	radius;								// radius
	
	Vector	position;							// position
	int		partsysid;							// particle system id
	float	partsysheight;						// particle system height
	int		num_of_particles;					// number of particles
		
	// grid
	
	GridCell*** gridcells;
	float cellsize;
	int max_x_cells;
	int max_y_cells;
	int max_z_cells;

	inline Container() {};
	Container(int i,Vector p,float w,float h,float d,bool o,bool hl,int psid,int pn,float psh,std::vector<Container*> subs);	

	// to grid
	inline float getMinX() {return position.x-radius;}
	inline float getMaxX() {return position.x+radius;}
	inline float getMinY() {return position.y-height/2;}
	inline float getMaxY() {return position.y+height/2;}
	inline float getMinZ() {return position.z-radius;}
	inline float getMaxZ() {return position.z+radius;}

	inline Fluid* getFluid() {return (Fluid*)partsystem;}
	inline int getFluidID() {return partsysid;}
	inline float getFluidVolume() {return width*partsysheight*depth;}

	void setCellSize(float* cs);
	void getSubNumber(int* num);									// summerize the containers
	void getSubs(Container** containers);							// collect the containers
	void initSystem(std::vector<ParticleSystem*> particlesystems);	// create the particles
	void createGrid();												// partition the container to cells
	void setGridCell(GridCell*** gc,float cs,int rid);

	void collision(Particle* particle,float diff,const Vector& n);
	void computeCollision(float rad,Particle* particle);
	void distribute();

	void move(Container** containers,int num);						// move container & partsystem
	void draw();
};

#endif /*CONTAINER_H*/
