// 
//    Distributed SPH Fluid Simulator (asterix@index.hu)
// =============================================================================
//
// Copyright 2007 David Illes
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 

#ifndef FLUIDSIM_H
#define FLUIDSIM_H

#include <stdio.h>			
#include <assert.h>
#include <stdlib.h>
#include <time.h>

#include "scene.hpp"

#include <GL/glut.h>

class FluidSimApp;

//-----------------------------------------------------------------------------
// fluidSim application class
//=============================================================================
class FluidSimApp {
//=============================================================================
private:
	double		loadingtime,calctime,fps_frame,fps;
	int			current_frame;
	
	bool grid_on;							// viewport grid switch
	bool text_on;							// viewport text switch
	
	clock_t		start;
	
	static		FluidSimApp* mInstance;

public:
	static FluidSimApp* Instance();			// application instance
	
	int rank;								// MPI application rank
	int size;								// MPI size
		
	Scene scene;							// scene description

	FluidSimApp();							
	void init();							// init application
	void initGL();							// init opengl
	void initLight();						// init light
	void initCamera();						// init camera
	void execute();							// execute application
	void newFrame();						// generate the new frame
	void simulate();						// simulate frames
	void createGrid(float step, float n);	// create viewport grid
	void toggleGrid();						// toggle grid
	void toggleText();						// toggle text
	void drawString(GLfloat x, GLfloat y, char *format,...); // draw string
	void display();							// update application
	void displayDatas();
	
};

#endif /*FLUIDSIM_H*/
