// 
//    Distributed SPH Fluid Simulator (asterix@index.hu)
// =============================================================================
//
// Copyright 2007 David Illes
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 

#ifndef PARTICLE_H
#define PARTICLE_H

#include "vector.hpp"
#include "neighbour.hpp"

//-----------------------------------------------------------------------------
// particle class
//=============================================================================
class Particle {
//=============================================================================
private:
	Vector	defposition;			// default position
public:
	inline 	Particle() { neighbours = new Neighbour[128]; }

	int		id;						// id
	int		lid;					// local id
	
	bool	isNull;
	bool	toThrow;	
	bool	received;			
	
	Vector	position;				// position
	Vector velocity;				// velocity
	float density;				// density
	float pressure;				// pressure
	
	float 	mass;					// particle mass
	
	int		cellx,celly,cellz;		// cell params
	int		cellid;					// cell id	
	
	int		num_of_neighbours;  	// number of neighbours
	
	Neighbour* neighbours;			// particle neighbours
	
	void init(int i,int li,float m,float x,float y,float z, float velx, float vely, float velz, bool rcv, bool mnull, bool mthrow);

	void resetNeighbours();
	void resetStatus();
	void resetPosition();
	
	void newNeighbour(Particle* p);	
};

#endif
