// 
//    Distributed SPH Fluid Simulator (asterix@index.hu)
// =============================================================================
//
// Copyright 2007 David Illes
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 

#include <vector>

#include "camera.hpp"
#include "particlesystem.hpp"
#include "container.hpp"

//-----------------------------------------------------------------------------
// scene description class
//=============================================================================
class Scene {
//=============================================================================

public:

	std::vector<ParticleSystem*> particlesystems;	// particle systems in the scene
	int num_of_particlesystems;
	std::vector<Container*> maincontainers;			// maincontainers in the scene
	int num_of_maincontainers;
	Container** containers;							// main & sub containers in the scene
	int num_of_containers;
	std::vector<Camera*> cameras;					// cameras in the scene
	int num_of_cameras;
	Camera* active_camera;							// the active camera
	Container* active_container;					// the active container
	ParticleSystem* active_particlesystem;			// the active particlesystem

	Scene();
	void initScene(const char* file);				// init the scene objects - from file
	void resetScene();								// reset the scene
	
	void newFrame();								// calculate new positions
	void switchCamera();							// switch to the next camera
	void switchParticleSystem();					// switch to the next particlesystem
	void display();									// draw the scene

};

