// 
//    Distributed SPH Fluid Simulator (asterix@index.hu)
// =============================================================================
//
// Copyright 2007 David Illes
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 

#include <math.h>

#ifndef VECTOR_H
#define VECTOR_H

//-----------------------------------------------------------------------------
// Vector class and overloaded operators
//==============================================================================
class Vector {
//==============================================================================
public:
	float x, y, z;							// coordinates		

	inline Vector() : x(0), y(0), z(0) { }


	inline double getX() {return x;}
	inline double getY() {return y;}
	inline double getZ() {return z;}
	inline void setX(double x0) {x=x0;}
	inline void setY(double y0) {y=y0;}
	inline void setZ(double z0) {z=z0;}

//------------------------------------------------------------------------------
inline Vector(float xx, float yy, float zz) {
//------------------------------------------------------------------------------
		x = xx; y = yy; z = zz;
}
	
//------------------------------------------------------------------------------
inline void Set(float xx, float yy, float zz) {
//------------------------------------------------------------------------------
		x = xx; y = yy; z = zz;
}

//------------------------------------------------------------------------------
inline void Set(const Vector& v) {
//------------------------------------------------------------------------------
		x = v.x; y = v.y; z = v.z;
}

//------------------------------------------------------------------------------
inline bool operator==(const Vector& c) const {
//------------------------------------------------------------------------------
	return(x == c.x && y == c.y && z == c.z);
}

//------------------------------------------------------------------------------
// binary operators
//------------------------------------------------------------------------------
inline Vector operator+(const Vector& c) const {
//------------------------------------------------------------------------------
		return Vector(x + c.x, y + c.y, z + c.z);
}

//------------------------------------------------------------------------------
inline Vector operator-(const Vector& c) const {
//------------------------------------------------------------------------------
		return Vector(x - c.x, y - c.y, z - c.z);
}

//------------------------------------------------------------------------------
inline Vector operator*(float f) const {
//------------------------------------------------------------------------------
		return Vector(x * f, y * f, z * f);
}

//------------------------------------------------------------------------------
inline Vector operator/(float f) const {
//------------------------------------------------------------------------------
		return Vector(x / f, y / f, z / f);
}

//------------------------------------------------------------------------------
// unary operators
//------------------------------------------------------------------------------
inline void operator+=(float f) { 
//------------------------------------------------------------------------------
		x += f; y += f; z += f; 
}

//------------------------------------------------------------------------------
inline void operator+=(const Vector& v) { 
//------------------------------------------------------------------------------
		x += v.x; y += v.y; z += v.z; 
}

//------------------------------------------------------------------------------
inline void operator=(const Vector& v) { 
//------------------------------------------------------------------------------
		x = v.x; y = v.y; z = v.z; 
}

//------------------------------------------------------------------------------
inline void operator-=(const Vector& v) { 
//------------------------------------------------------------------------------
		x -= v.x; y -= v.y; z -= v.z; 
}

//------------------------------------------------------------------------------
inline void operator-=(float f) { 
//------------------------------------------------------------------------------
		x -= f; y -= f; z -= f; 
}

//------------------------------------------------------------------------------
inline void operator*=(float f) { 
//------------------------------------------------------------------------------
		x *= f; y *= f; z *= f; 
}

//------------------------------------------------------------------------------
inline void operator*=(const Vector& v) { 
//------------------------------------------------------------------------------
		x *= v.x; y *= v.y; z *= v.z; 
}

//------------------------------------------------------------------------------
inline void operator/=(float f) { 
//------------------------------------------------------------------------------
		x /= f; y /= f; z /= f; 
}

//------------------------------------------------------------------------------
inline void operator/=(const Vector& v) { 
//------------------------------------------------------------------------------
		x /= v.x; y /= v.y; z /= v.z; 
}

//------------------------------------------------------------------------------
inline Vector operator-(void) const { 
//------------------------------------------------------------------------------
		return Vector(-x, -y, -z);
}

//------------------------------------------------------------------------------
// dot product
//------------------------------------------------------------------------------
inline float operator*(const Vector& v) const {
//------------------------------------------------------------------------------
		return x * v.x + y * v.y + z * v.z; 
}

//------------------------------------------------------------------------------
// cross product
//------------------------------------------------------------------------------
inline Vector operator%(const Vector& v) const {
//------------------------------------------------------------------------------
		return Vector(y * v.z - z * v.y, z * v.x - x * v.z, x * v.y - y * v.x);
}

//------------------------------------------------------------------------------
inline Vector operator*(float f) {
//------------------------------------------------------------------------------
	return Vector(f * x, f * y, f * z);
}


//------------------------------------------------------------------------------
inline float Norm(void) const {
//------------------------------------------------------------------------------
		return sqrt(x*x + y*y + z*z);
}
//------------------------------------------------------------------------------
inline float Norm_2(void) const {
//------------------------------------------------------------------------------
		return x*x + y*y + z*z;
}

//------------------------------------------------------------------------------
inline float Dot(const Vector& v) const {
//------------------------------------------------------------------------------
		return (x*v.x + y*v.y + z*v.z);
}

//------------------------------------------------------------------------------
inline void Normalize() {
//------------------------------------------------------------------------------
		float norm = Norm();
		x /= norm;
		y /= norm;
		z /= norm;
}

//------------------------------------------------------------------------------
inline float dist(Vector& v) {
//------------------------------------------------------------------------------
	return (*this-v).Norm();
}

};

#endif /*VECTOR_H*/
