//      ___                       ___           ___           ___       ___     
//     /\__\          ___        /\__\         /\  \         /\__\     /\  \.   
//    /::|  |        /\  \      /::|  |       /::\  \       /:/  /    /::\  \.  
//   /:|:|  |        \:\  \    /:|:|  |      /:/\:\  \     /:/  /    /:/\:\  \. 
//  /:/|:|__|__      /::\__\  /:/|:|  |__   /:/  \:\  \   /:/  /    /::\~\:\  \.
// /:/ |::::\__\  __/:/\/__/ /:/ |:| /\__\ /:/__/_\:\__\ /:/__/    /:/\:\ \:\__\.
// \/__/~~/:/  / /\/:/  /    \/__|:|/:/  / \:\  /\ \/__/ \:\  \    \:\~\:\ \/__/
//       /:/  /  \::/__/         |:/:/  /   \:\ \:\__\    \:\  \    \:\ \:\__\. 
//      /:/  /    \:\__\         |::/  /     \:\/:/  /     \:\  \    \:\ \/__/  
//     /:/  /      \/__/         /:/  /       \::/  /       \:\__\    \:\__\.   
//     \/__/                     \/__/         \/__/         \/__/     \/__/    
// 
// =============================================================================
//                       Minimalist OpenGL Environment
//                       Parallel Rendering Extension
// =============================================================================
//
// Copyright 2007 Balazs Domonkos
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file ParallelRenderingSupport.h
/// General parallel rendering support manager class

#ifndef __MINGLE_PARALLEL_PARALLEL_RENDERING_SUPPORT_H__
#define __MINGLE_PARALLEL_PARALLEL_RENDERING_SUPPORT_H__

// Include class definitions
#include "parallel-classes.h"
#include "ParallelHelpers.h"

#include <vector>

namespace MinGLE {
	class ParallelRenderingSupport : public SystemListener {
		public:
	    	enum SupportType {
				UNKNOWN,
				PARACOMP
	    	};

	    	enum SystemPart {
				PEER2PEER,
				MASTER,
				SLAVE
	    	};

			enum CompositingOperator { 
				DEPTH_OPERATOR, 
				ALPHA_BLEND_OPERATOR 
			};

			/// Display debug constants
			enum DisplayDebug {
				DISPLAY_FILL 			= 0x1, 
				DISPLAY_NODE_NAME		= 0x2, 
				DISPLAY_BOUNDING_RECT 	= 0x4, 
				DISPLAY_BOUNDING_BOX 	= 0x8 
			};
		
		protected:
			static const CompositingOperator DEFAULT_COMPOSITING_OPERATOR = DEPTH_OPERATOR;

		protected:
	    	static SupportType msSupportType;
	    	static ParallelRenderingSupport *msInstance;

		protected:
	    	ParallelRenderingSupport();
	    	virtual ~ParallelRenderingSupport();
	    
	    	virtual void _addParallelSupport(Window *window, CompositingOperator compositingOperator, SystemPart part) = 0;
	    	virtual void _setParameter(const unsigned type, const void *value) = 0;
	    	virtual int _getRendererCount(Window *window) = 0;
	    	virtual int _getThisRenderer(Window *window) = 0;
	    	virtual void _setPartition(Window *window, BoxPartitioner<double>::Partition *partition) = 0;
	    	virtual void _setSortOrder(Window *window, unsigned order) = 0;
	    	virtual unsigned _getDisplayDebug(Window *window) = 0;
	    	virtual void _setDisplayDebug(Window *window, const unsigned displayDebug) = 0;
	    	virtual void _toggleDisplayDebug(Window *window, const DisplayDebug flag) = 0;
	    	//virtual void _setBoundingBox(Window *window, const double *offset, const double *size) = 0;
	    	virtual SystemPart _getSystemPart(Window *window) const = 0;
	    	virtual void _getBlendingFactors(const CompositingOperator op, unsigned& sfactorRGB, unsigned& dfactorRGB, unsigned& sfactorAlpha, unsigned& dfactorAlpha) const = 0;
	    
	    public:
	    	static void initialize(int *argc, char **argv, SupportType supportType = UNKNOWN);
	    	static void initialize(unsigned sessionId, std::vector<std::string> hostList, SupportType supportType = UNKNOWN);
	    	static void addParallelSupport(Window *window, CompositingOperator compositingOperator = DEFAULT_COMPOSITING_OPERATOR, SystemPart part = PEER2PEER);
	    	inline static void setParameter(const unsigned type, const void *value) {
	    		msInstance->_setParameter(type, value);
	    	}
	    	inline static int getRendererCount(Window *window) {
	    		return msInstance->_getRendererCount(window);
	    	}
	    	inline static int getThisRenderer(Window *window) {
	    		return msInstance->_getThisRenderer(window);
	    	}
	    	inline static void setPartition(Window *window, BoxPartitioner<double>::Partition *partition) {
	    		msInstance->_setPartition(window, partition);
	    	}
	    	inline static void setSortOrder(Window *window, unsigned order) {
	    		msInstance->_setSortOrder(window, order);
	    	}
	    	inline static unsigned getDisplayDebug(Window *window) {
	    		return msInstance->_getDisplayDebug(window);
	    	}
	    	inline static void setDisplayDebug(Window *window, const unsigned displayDebug) {
	    		msInstance->_setDisplayDebug(window, displayDebug);
	    	}
	    	inline static void toggleDisplayDebug(Window *window, const DisplayDebug flag) {
	    		msInstance->_toggleDisplayDebug(window, flag);
	    	}
	    	/*inline static void setBoundingBox(Window *window, const double *offset, const double *size) {
	    		msInstance->_setBoundingBox(window, offset, size);
	    	}*/
	    	inline static SystemPart getSystemPart(Window *window) {
	    		return msInstance->_getSystemPart(window);
	    	}
	    	inline static void getBlendingFactors(const CompositingOperator op, unsigned& sfactorRGB, unsigned& dfactorRGB, unsigned& sfactorAlpha, unsigned& dfactorAlpha) {
	    		msInstance->_getBlendingFactors(op, sfactorRGB, dfactorRGB, sfactorAlpha, dfactorAlpha);
	    	}
	    	
	    	static std::string getHostName();
	    	inline static bool needSorting(CompositingOperator compositingOperator) {
	    		// Wired logic for performance
	    		return compositingOperator == ALPHA_BLEND_OPERATOR;
	    	}
	    	
	    	/// Get unique color for current window
	    	static void getUniqueColor(Window *window, double &r, double &g, double &b);

		protected:
			static void _parseCommandLine(int *argc, char **argv, unsigned &sessionId, std::vector<std::string> &hostList);
			static void _initializeParaComp(unsigned sessionId, std::vector<std::string> hostList);
			static ParallelRenderingWindowProxy *_findParallelRenderingWindowProxy(Window *window); 
	};
}

#endif // __MINGLE_PARALLEL_PARALLEL_RENDERING_SUPPORT_H__
