//      ___                       ___           ___           ___       ___     
//     /\__\          ___        /\__\         /\  \         /\__\     /\  \.   
//    /::|  |        /\  \      /::|  |       /::\  \       /:/  /    /::\  \.  
//   /:|:|  |        \:\  \    /:|:|  |      /:/\:\  \     /:/  /    /:/\:\  \. 
//  /:/|:|__|__      /::\__\  /:/|:|  |__   /:/  \:\  \   /:/  /    /::\~\:\  \.
// /:/ |::::\__\  __/:/\/__/ /:/ |:| /\__\ /:/__/_\:\__\ /:/__/    /:/\:\ \:\__\.
// \/__/~~/:/  / /\/:/  /    \/__|:|/:/  / \:\  /\ \/__/ \:\  \    \:\~\:\ \/__/
//       /:/  /  \::/__/         |:/:/  /   \:\ \:\__\    \:\  \    \:\ \:\__\. 
//      /:/  /    \:\__\         |::/  /     \:\/:/  /     \:\  \    \:\ \/__/  
//     /:/  /      \/__/         /:/  /       \::/  /       \:\__\    \:\__\.   
//     \/__/                     \/__/         \/__/         \/__/     \/__/    
// 
// =============================================================================
//                       Minimalist OpenGL Environment
//                       Parallel Rendering Extension
// =============================================================================

//
// Original source of this class: HP XC ParaComp Kit.
//
// Copyright 2007 Balazs Domonkos
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file SimpleCommSock.h
/// Simple socket based communication

#ifndef __MINGLE_PARALLEL_SIMPLE_COMM_SOCK_H__
#define __MINGLE_PARALLEL_SIMPLE_COMM_SOCK_H__

// Include class definitions
#include "parallel-classes.h"

#include <string>
#include <vector>
#include <map>

namespace MinGLE {
	class SimpleCommSock {

		protected:
			static const int MAX_CONNECTIONS = 64;

		private:
        	char mMsgBuffer[1024];
			std::vector<int> mHostSock;
			std::map<std::string, int> mHostToIndex;
            int mMySock;

		public:
			SimpleCommSock();
			~SimpleCommSock();
			
			// Master waiting for slaves to connect
			void waitHosts(const std::vector<std::string>& hosts, const std::string& me, unsigned port);
			
			// Slaves connecting to master
			void connect(const std::string& me, const std::string& master, unsigned port);
			
			// From master
			void broadcastMessage(void *data, size_t size);
			
			// Slave use only
			void* recvMessage(unsigned size);
			
			// Close connection
			void close();

		private:
			void readSock(int sock, void *buffer, unsigned int nBytes);
            void writeSock(int sock, void *buffer, unsigned int nBytes);

            void cleanup();
            void setOpts();
	};
}

#endif // __MINGLE_PARALLEL_SIMPLE_COMM_SOCK_H__
