//      ___                       ___           ___           ___       ___     
//     /\__\          ___        /\__\         /\  \         /\__\     /\  \.   
//    /::|  |        /\  \      /::|  |       /::\  \       /:/  /    /::\  \.  
//   /:|:|  |        \:\  \    /:|:|  |      /:/\:\  \     /:/  /    /:/\:\  \. 
//  /:/|:|__|__      /::\__\  /:/|:|  |__   /:/  \:\  \   /:/  /    /::\~\:\  \.
// /:/ |::::\__\  __/:/\/__/ /:/ |:| /\__\ /:/__/_\:\__\ /:/__/    /:/\:\ \:\__\.
// \/__/~~/:/  / /\/:/  /    \/__|:|/:/  / \:\  /\ \/__/ \:\  \    \:\~\:\ \/__/
//       /:/  /  \::/__/         |:/:/  /   \:\ \:\__\    \:\  \    \:\ \:\__\. 
//      /:/  /    \:\__\         |::/  /     \:\/:/  /     \:\  \    \:\ \/__/  
//     /:/  /      \/__/         /:/  /       \::/  /       \:\__\    \:\__\.   
//     \/__/                     \/__/         \/__/         \/__/     \/__/    
// 
// =============================================================================
//                       Minimalist OpenGL Environment
//                       Parallel Rendering Extension
// =============================================================================
//
// Copyright 2007 Balazs Domonkos
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file SlavePart.h
/// ParaComp slave part window proxy

#ifndef __MINGLE_PARALLEL_PARACOMP_SLAVEPART_H__
#define __MINGLE_PARALLEL_PARACOMP_SLAVEPART_H__

// Include class definitions
#include <parallel-classes.h>
#include <paracomp/include/Part.h>

#include <string>

#if MINGLE_PARACOMP_SUPPORT == 1

namespace MinGLE {
	class ParaCompSlavePart : public ParaCompPart {
		protected:
			std::string mThisHost;
			std::string mMasterHost;
			
			EventInjector *mEventInjector;

	    public:
	    	ParaCompSlavePart(Window *window, const std::string& thisHost, ParaCompSupport *support, ParallelRenderingSupport::CompositingOperator compositingOperator);
	    	virtual ~ParaCompSlavePart();

	    	virtual void createContext();

			// Override render method
    		inline virtual bool onRender() {
    			// Before rendering a frame the queued events received from
    			// the event collector are injected.
    			// The window listeners attached to this window proxy will
    			// get these events as if they were generated by the user.
    			mEventInjector->injectQueuedEvents();

				// Do not start new frame when the window is closing.
				// Starting a new but unfinished frame will freeze the program.    			
    			if(!mWindow->isClosing()) {

        	    	// Contributepixels and retrieve output
        	    	pixelTransfer();
    			}
    				
				return false;
    		}

		protected:
			virtual void _refreshContextInfo();
	};
}

#endif // MINGLE_PARACOMP_SUPPORT == 1

#endif // __MINGLE_PARALLEL_PARACOMP_SLAVEPART_H__
