//      ___                       ___           ___           ___       ___     
//     /\__\          ___        /\__\         /\  \         /\__\     /\  \.   
//    /::|  |        /\  \      /::|  |       /::\  \       /:/  /    /::\  \.  
//   /:|:|  |        \:\  \    /:|:|  |      /:/\:\  \     /:/  /    /:/\:\  \. 
//  /:/|:|__|__      /::\__\  /:/|:|  |__   /:/  \:\  \   /:/  /    /::\~\:\  \.
// /:/ |::::\__\  __/:/\/__/ /:/ |:| /\__\ /:/__/_\:\__\ /:/__/    /:/\:\ \:\__\.
// \/__/~~/:/  / /\/:/  /    \/__|:|/:/  / \:\  /\ \/__/ \:\  \    \:\~\:\ \/__/
//       /:/  /  \::/__/         |:/:/  /   \:\ \:\__\    \:\  \    \:\ \:\__\. 
//      /:/  /    \:\__\         |::/  /     \:\/:/  /     \:\  \    \:\ \/__/  
//     /:/  /      \/__/         /:/  /       \::/  /       \:\__\    \:\__\.   
//     \/__/                     \/__/         \/__/         \/__/     \/__/    
// 
// =============================================================================
//                       Minimalist OpenGL Environment
//                       Parallel Rendering Extension
// =============================================================================
//
// Copyright 2007 Balazs Domonkos
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file Support.h
/// ParaComp parallel rendering support

#ifndef __MINGLE_PARALLEL_PARACOMP_SUPPORT_H__
#define __MINGLE_PARALLEL_PARACOMP_SUPPORT_H__

// Include class definitions
#include <parallel-classes.h>
#include <ParallelRenderingSupport.h>
#include <ParallelHelpers.h>
#include <paracomp/include/Part.h>

#include <string>
#include <vector>

#if MINGLE_PARACOMP_SUPPORT == 1

namespace MinGLE {
	class ParaCompSupport : public ParallelRenderingSupport {
		friend class ParallelRenderingSupport;

		protected:
			unsigned mSessionId;
			unsigned mNetId;
			std::vector<std::string> mHostList;

		public:
			inline unsigned getNetId() {
				return mNetId; 
			}

		protected:
			ParaCompSupport(unsigned sessionId, std::vector<std::string> hostlist);
			virtual ~ParaCompSupport();

		   	virtual void _addParallelSupport(Window *window, ParallelRenderingSupport::CompositingOperator compositingOperator, SystemPart part);
		   	virtual void _addParallelSupportMaster(Window *window, ParallelRenderingSupport::CompositingOperator compositingOperator);
		   	virtual void _addParallelSupportSlave(Window *window, ParallelRenderingSupport::CompositingOperator compositingOperator);

		   	virtual void _setParameter(const unsigned type, const void *value);
	    	virtual int _getRendererCount(Window *window);
	    	virtual int _getThisRenderer(Window *window);
	    	virtual void _setPartition(Window *window, BoxPartitioner<double>::Partition *partition);
	    	virtual void _setSortOrder(Window *window, unsigned order);
	    	virtual unsigned _getDisplayDebug(Window *window);
	    	virtual void _setDisplayDebug(Window *window, const unsigned displayDebug);
	    	virtual void _toggleDisplayDebug(Window *window, const ParallelRenderingSupport::DisplayDebug flag);
	    	//virtual void _setBoundingBox(Window *window, const double *offset, const double *size);
	    	virtual SystemPart _getSystemPart(Window *window) const;
	    	virtual void _getBlendingFactors(const CompositingOperator op, unsigned& sfactorRGB, unsigned& dfactorRGB, unsigned& sfactorAlpha, unsigned& dfactorAlpha) const;
	    	
	    	/// Step up along the window proxy chain and try to find a 
	    	/// Window which has a window proxy with type ParaCompPart. 
	    	virtual ParaCompPart* _findProxiedWindow(Window *window) const;
	};
}

#endif // MINGLE_PARACOMP_SUPPORT == 1

#endif // __MINGLE_PARALLEL_PARACOMP_SUPPORT_H__
