dnl
dnl  This file is part of Minimalist OpenGL Environment (MinGLE)
dnl 
dnl  Version: 0.2.0
dnl  Author: Balazs Domonkos
dnl  Filename: acinclude.m4
dnl  Creation Date: November 16th 2007
dnl  Revision Date: November 16th 2007
dnl 
dnl 
dnl  The Minimalist OpenGL Environment is free software: you can
dnl  redistribute it and/or modify it under the terms of the GNU
dnl  General Public License as published by the Free Software
dnl  Foundation, either version 3 of the License, or (at your
dnl  option) any later version.
dnl 
dnl  The Minimalist OpenGL Environment is distributed in the hope
dnl  that it will be useful, but WITHOUT ANY WARRANTY; without
dnl  even the implied warranty of MERCHANTABILITY or FITNESS FOR
dnl  A PARTICULAR PURPOSE. See the GNU General Public License
dnl  for more details.
dnl 
dnl  You should have received a copy of the GNU General Public License
dnl  along with Minimalist OpenGL Environment.  If not, see 
dnl  <http://www.gnu.org/licenses/>. See the COPYING file included 
dnl  with this distribution file for license details.
dnl

dnl 
dnl Detect target OS, setup compiler flags and OpenGL libraries
dnl 
AC_DEFUN([MINGLE_DETECT_TARGET],
[case $host in
  *-*-cygwin* | *-*-mingw* | *-*-pw32*)
	AC_SUBST(SHARED_FLAGS, "-shared -no-undefined -Xlinker --export-all-symbols")
	AC_SUBST(PLUGIN_FLAGS, "-shared -no-undefined -avoid-version")
	AC_SUBST(GL_LIBS, "-lopengl32 -lglu32")	
	MINGLE_TARGET=nt
    	AC_MSG_NOTICE([NT target OS detected ($host)])
	;;
  *-*-darwin*)
        AC_SUBST(SHARED_FLAGS, "-shared")
        AC_SUBST(PLUGIN_FLAGS, "-shared -avoid-version")
        AC_SUBST(GL_LIBS, "-lGL -lGLU")
	MINGLE_TARGET=osx
    	AC_MSG_NOTICE([OSX target OS detected ($host)])
	;;
  *) dnl default to standard linux
	AC_SUBST(SHARED_FLAGS, "-shared")
	AC_SUBST(PLUGIN_FLAGS, "-shared -avoid-version")
	AC_SUBST(GL_LIBS, "-lGL -lGLU")
	MINGLE_TARGET=linux
    	AC_MSG_NOTICE([default target OS is used, which is linux ($host)])
	;;
esac
dnl you must arrange for every AM_conditional to run every time configure runs
AM_CONDITIONAL(MINGLE_TARGET_NT, test $MINGLE_TARGET = nt)
AM_CONDITIONAL(MINGLE_TARGET_OSX,test  $MINGLE_TARGET = osx )
AM_CONDITIONAL(MINGLE_TARGET_LINUX, test $MINGLE_TARGET = linux)
])

dnl 
dnl Detect target architecture
dnl 
AC_DEFUN([MINGLE_DETECT_ARCH],
[
	case $host in
		x86_64*)
			MINGLE_ARCH=x86_64
			AC_MSG_NOTICE([64-bit target architecture detected ($host)])
			;;
		  *) dnl default for '*i[[3456]]86*)'
			MINGLE_ARCH=x86
			AC_MSG_NOTICE([32-bit target architecture detected ($host)])
		  	;;
	esac
])

dnl
dnl Detect endianness
dnl
AC_DEFUN([MINGLE_DETECT_ENDIANNES],
[AC_TRY_RUN([
		int main()
		{
			short s = 1;
			short* ptr = &s;
			unsigned char c = *((char*)ptr);
			return c;
		}
	], MINGLE_ENDIANNESS=big, MINGLE_ENDIANNESS=little)
])

dnl
dnl Setup compiler options
dnl
AC_DEFUN([MINGLE_COMPILER_OPTIONS],
[
	dnl Set endianness
	case $MINGLE_ENDIANNESS in
		big)
			CXXFLAGS="$CXXFLAGS -DMINGLE_BIG_ENDIAN"
			CFLAGS="$CFLAGS -DMINGLE_BIG_ENDIAN"
			;;
		little)
			CXXFLAGS="$CXXFLAGS -DMINGLE_LITTLE_ENDIAN"
			CFLAGS="$CFLAGS -DMINGLE_LITTLE_ENDIAN"
			;;
	esac

	dnl Enabling or disabling automatic CPU optimalization
        AC_ARG_ENABLE(cpuopt,
	    AC_HELP_STRING([--disable-cpuopt], [Disable automatic CPU optimalization (default enabled)]),
	    [MINGLE_CPU_OPT=$enableval],
	    [MINGLE_CPU_OPT=yes])
	AM_CONDITIONAL(MINGLE_CPU_OPT, test $MINGLE_CPU_OPT = yes)

	dnl Try to get the best gcc cpu optimalization flags for the current cpu
	./gcccpuopt 2>&1 | grep "Your compiler supports the -march=native option" && gcccpuopt_native=yes
	gcccpuopt=`./gcccpuopt`

	dnl Fix this result	
	case $MINGLE_ARCH in
		x86)
			# nothing to do
			;;
		x86_64)
			# tune to 64 bit
			gcccpuopt=`echo "${gcccpuopt}" | sed "s|-m32|-m64|g"`
			
			# check the operation of the compiler using the modified settings
			compiles_64bit=`${CC} ${gcccpuopt} -S -o /dev/null -xc /dev/null >/dev/null 2>&1 && echo yes`
			if test "x${compiles_64bit}" != xyes; then
			    if test "x${gcccpuopt_native}" = xyes; then
				gcccpuopt=`echo "${gcccpuopt}" | sed "s|-march=.* |-march=native |g"`
			    else
				AC_MSG_ERROR([gcccpuopt supports only 32bit architectures. However, I have tried to tune the flags for 64bit. Unfortunately, your compiler does not support neither the tuned flags (${gcccpuopt}) nor -march=native. Try to run the configuration script with setting the --disable-cpuopt flag to avoid this situation.])
			    fi
			fi
			;;
	esac
	CFLAGS="$CFLAGS ${gcccpuopt}"
	CXXFLAGS="$CXXFLAGS ${gcccpuopt}"	

	dnl Set warning flags
	CFLAGS="$CFLAGS -Wall"
	CXXFLAGS="$CXXFLAGS -Wall"	
])

dnl
dnl Setup additional include paths
dnl
AC_DEFUN([MINGLE_ADDITIONAL_INCLUDE_PATHS],
[
	AC_MSG_CHECKING(for additional include paths)
	AC_ARG_WITH(incpath,
		[  --with-incpath=PATH     add PATH0:PATH1:.. to the include path],
		CFLAGS="$CFLAGS -I"`echo "$with_incpath" | sed "s|:| -I|g"`
		CXXFLAGS="$CXXFLAGS -I"`echo "$with_incpath" | sed "s|:| -I|g"`
		x="$with_incpath"
	,
		x="none particular"
	)
	AC_MSG_RESULT([$x])
])

dnl
dnl Setup additional library paths
dnl
AC_DEFUN([MINGLE_ADDITIONAL_LIBRARY_PATHS],
[
	AC_MSG_CHECKING(for additional library dir)
	AC_ARG_WITH(libpath,
		[  --with-libpath=PATH     add PATH0:PATH1:.. to the library path],
		LDFLAGS="$LDFLAGS -L"`echo "$with_libpath" | sed "s|:| -L|g"`
		x="$with_libpath"
	,
		x="none particular"
	)
	AC_MSG_RESULT([$x])
])

dnl 
dnl Check for standard OpenGL headers and libraries
dnl 
AC_DEFUN([MINGLE_CHECK_OPENGL_STANDARD],
[
	AC_MSG_CHECKING(for standard OpenGL headers and libraries)
	AC_MSG_RESULT()
	AC_CHECK_HEADERS(GL/gl.h, [], AC_MSG_ERROR([Header 'GL/gl.h' is not found]))
	AC_CHECK_HEADERS(GL/glu.h, [], AC_MSG_ERROR([Header 'GL/glu.h' is not found]))
	AC_CHECK_LIB(GL, main, [], AC_MSG_ERROR([Library 'GL' is not found]))
	AC_CHECK_LIB(GLU, main, [], AC_MSG_ERROR([Library 'GLU' is not found]))
])

dnl 
dnl Check for OpenGL extension headers, libraries, and wrappers
dnl 
AC_DEFUN([MINGLE_CHECK_OPENGL_EXTENSIONS],
[
	AC_MSG_CHECKING(for OpenGL extension headers, libraries, and wrappers)
	AC_MSG_RESULT()

	dnl Checking OpenGL Extension Wrangler support
	AC_MSG_CHECKING([whether to build OpenGL Extension Wrangler support])
	AC_ARG_WITH(glew,[  --with-glew=yes|no|auto 
                          build OpenGL Extension Wrangler support (default is auto)],
		MINGLE_GLEW=$withval
	,
		MINGLE_GLEW=auto
	)
	AC_MSG_RESULT([$MINGLE_GLEW])

	dnl Checking headers, libraries, and wrappers
	case $MINGLE_GLEW in
		yes)
			AC_CHECK_HEADERS(GL/glew.h, [], AC_MSG_ERROR([Header 'GL/glew.h' is not found]))
			AC_CHECK_LIB(GLEW, main, [], AC_MSG_ERROR([Library 'GLEW' is not found]))
			;;
		no)
			;;
		auto)
			MINGLE_GLEW=yes
			AC_CHECK_HEADERS(GL/glew.h, [], MINGLE_GLEW=no)
			AC_CHECK_LIB(GLEW, main, [], MINGLE_GLEW=no)
			;;
		*)
			AC_MSG_ERROR([Invalid option '$MINGLE_GLEW'. Use yes, no, or auto])
	esac

	dnl Set results for automake
	AM_CONDITIONAL(MINGLE_GLEW, test $MINGLE_GLEW = yes)
	
	dnl Write result to config.h
	if test $MINGLE_GLEW = yes; then
    	    AC_DEFINE([MINGLE_GLEW], [], [Build GLEW OpenGL Extension Wrangler support])
	fi
])

dnl 
dnl Check for OpenGL shading language headers, libraries, and wrappers
dnl 
AC_DEFUN([MINGLE_CHECK_SHADING],
[
	MINGLE_SHADING_LANGUAGES=""

	AC_MSG_CHECKING(for OpenGL shading language headers, libraries, and wrappers)
	AC_MSG_RESULT()

	dnl Checking Cg support
	AC_MSG_CHECKING([whether to build Cg support])
	AC_ARG_WITH(cg,
		[  --with-cg=yes|no|auto   build OpenGL Cg support (default is auto)],
		MINGLE_CG=$withval
	,
		MINGLE_CG=auto
	)
	AC_MSG_RESULT([$MINGLE_CG])

	dnl Checking headers, libraries, and wrappers
	case $MINGLE_CG in
		yes)
			dnl On linux target pthread library is needed for Cg
			if test $MINGLE_TARGET = linux; then
				AC_CHECK_LIB(pthread, main, [], [])
			fi

			AC_CHECK_HEADERS(Cg/cg.h, [], AC_MSG_ERROR([Header 'Cg/cg.h' is not found]))
			AC_CHECK_HEADERS(Cg/cgGL.h, [], AC_MSG_ERROR([Header 'Cg/cgGL.h' is not found]))
			AC_CHECK_LIB(libCg, main, [], AC_MSG_ERROR([Library 'libCg' is not found]))
			AC_CHECK_LIB(libCgGL, main, [], AC_MSG_ERROR([Library 'libCgGL' is not found]))
			;;
		no)
			;;
		auto)
			dnl On linux target pthread library is needed for Cg
			if test $MINGLE_TARGET = linux; then
				AC_CHECK_LIB(pthread, main, [], [])
			fi

			MINGLE_CG=yes
			AC_CHECK_HEADERS(Cg/cg.h, [], MINGLE_CG=no)
			AC_CHECK_HEADERS(Cg/cgGL.h, [], MINGLE_CG=no)
			AC_CHECK_LIB(Cg, main, [], MINGLE_CG=no)
			AC_CHECK_LIB(CgGL, main, [], MINGLE_CG=no)
			;;
		*)
			AC_MSG_ERROR([Invalid option '$MINGLE_CG'. Use yes, no, or auto])
	esac

	dnl Set results for automake
	AM_CONDITIONAL(MINGLE_CG, test $MINGLE_CG = yes)
	if test "$MINGLE_CG" = "yes"; then
		MINGLE_SHADING_LANGUAGES="$MINGLE_SHADING_LANGUAGES Cg"
	fi

	dnl TODO: glsl
])

dnl 
dnl Detect available render systems on target
dnl 
AC_DEFUN([MINGLE_DETECT_RENDER_SYSTEM],
[
	MINGLE_RENDER_SYSTEMS=""

	dnl Checking GLUT rendering system
	AC_MSG_CHECKING([whether to build GLUT rendering system])
	AC_ARG_WITH(glut,[  --with-glut=yes|no|auto 
                          build GLUT rendering support (default is auto)],
		glut_enable=$withval
	,
		glut_enable=auto
	)
	AC_MSG_RESULT([$glut_enable])

	dnl Checking headers and libraries
	case $glut_enable in
		yes)
			AC_CHECK_HEADERS(GL/glut.h, [], AC_MSG_ERROR([Header 'GL/glut.h' is not found]))
			AC_CHECK_LIB(glut, main, [], AC_MSG_ERROR([Library 'glut' is not found]))
			;;
		no)
			;;
		auto)
			glut_enable=yes
			AC_CHECK_HEADERS(GL/glut.h, [], glut_enable=no)
			AC_CHECK_LIB(glut, main, [], glut_enable=no)
			;;
		*)
			AC_MSG_ERROR([Invalid option '$glut_enable'. Use yes, no, or auto])
	esac

	dnl Set results for automake and for configure summary table
	AM_CONDITIONAL(MINGLE_RENDER_SYSTEM_GLUT, test $glut_enable = yes)
	if test "$glut_enable" = "yes"; then
		MINGLE_RENDER_SYSTEMS="$MINGLE_RENDER_SYSTEMS GLUT"
	fi

	dnl TODO: glx, sdl and wingl
	
	if test "x$MINGLE_RENDER_SYSTEMS" = x; then
	    AC_MSG_ERROR([None of supported OpenGL render systems was found. Please note that the currently supported render system is GLUT.])
	fi
])

dnl 
dnl Detect threading support on target
dnl 
AC_DEFUN([MINGLE_DETECT_THREADING],
[
	dnl Setting threading support (PTH)
	AC_MSG_CHECKING([set threading support (pth)])
	AC_ARG_WITH(threading_pth,[  --with-pth=yes|no|auto 
                          set type of pth threading support (default is auto)],
		MINGLE_THREADING_PTH=$withval
	,
		MINGLE_THREADING_PTH=auto
	)
	AC_MSG_RESULT([$MINGLE_THREADING_PTH])

	dnl Checking headers and libraries (PTH)
	case $MINGLE_THREADING_PTH in
		yes)
			AC_CHECK_HEADERS(pth.h, [], AC_MSG_ERROR([Header 'pth.h' is not found]))
			AC_CHECK_LIB(pth, main, [], AC_MSG_ERROR([Library 'pth' is not found]))
			;;
		no)
			;;
		auto)
			MINGLE_THREADING_PTH=yes
			AC_CHECK_HEADERS(pth.h, [], MINGLE_THREADING_PTH=no)
			AC_CHECK_LIB(pth, main, [], MINGLE_THREADING_PTH=no)
			;;
		*)
			AC_MSG_ERROR([Invalid option '$MINGLE_THREADING_PTH'. Use yes, no, or auto])
	esac


	dnl Setting threading support (PTHREAD)
	AC_MSG_CHECKING([set threading support (pthread)])
	AC_ARG_WITH(threading_pthread,[  --with-pthread=yes|no|auto 
                          set type of pthread threading support (default is auto)],
		MINGLE_THREADING_PTHREAD=$withval
	,
		MINGLE_THREADING_PTHREAD=auto
	)
	AC_MSG_RESULT([$MINGLE_THREADING_PTHREAD])

	dnl Checking headers and libraries (PTHREAD)
	case $MINGLE_THREADING_PTHREAD in
		yes)
			AC_CHECK_HEADERS(pthread.h, [], AC_MSG_ERROR([Header 'pthread' is not found]))
			AC_CHECK_LIB(pthread, main, [], AC_MSG_ERROR([Library 'pthread' is not found]))
			;;
		no)
			;;
		auto)
			MINGLE_THREADING_PTHREAD=yes
			AC_CHECK_HEADERS(pthread.h, [], MINGLE_THREADING_PTHREAD=no)
			AC_CHECK_LIB(pthread, main, [], MINGLE_THREADING_PTHREAD=no)
			;;
		*)
			AC_MSG_ERROR([Invalid option '$MINGLE_THREADING_PTHREAD'. Use yes, no, or auto])
	esac


	dnl TODO threading on nt systems

	dnl If no threading support is found raise error
	if test "$MINGLE_THREADING_PTH" = "no" -a "$MINGLE_THREADING_PTHREAD" = "no"; then
		AC_MSG_ERROR([No threading support is found on target system. Note that pth and pthread are supported.])
	fi

	dnl Set results for automake
	AM_CONDITIONAL(MINGLE_THREADING_PTH, test $MINGLE_THREADING_PTH = yes)
	AM_CONDITIONAL(MINGLE_THREADING_PTHREAD, test $MINGLE_THREADING_PTHREAD = yes)

	dnl Concetenate info string
	MINGLE_THREADING=""
	if test "$MINGLE_THREADING_PTH" = "yes"; then
		MINGLE_THREADING="$MINGLE_THREADING pth"
	fi
	if test "$MINGLE_THREADING_PTHREAD" = "yes"; then
		MINGLE_THREADING="$MINGLE_THREADING pthread"
	fi
])

dnl 
dnl Detect imaging support on target
dnl 
AC_DEFUN([MINGLE_DETECT_IMAGING],
[
	dnl Setting imaging support (freeimage)
	AC_MSG_CHECKING([set imaging support (freeimage)])
	AC_ARG_WITH(imaging_freeimage,[  --with-imaging-freeimage=yes|no|auto 
                          set type of freeimage imaging support (default is auto)],
		MINGLE_IMAGING_FREEIMAGE=$withval
	,
		MINGLE_IMAGING_FREEIMAGE=auto
	)
	AC_MSG_RESULT([$MINGLE_IMAGING_FREEIMAGE])

	dnl Checking headers and libraries (freeimage)
	case $MINGLE_IMAGING_FREEIMAGE in
		yes)
			AC_CHECK_HEADERS(FreeImage.h, [], AC_MSG_ERROR([Header 'FreeImage.h' is not found]))
			AC_CHECK_LIB(freeimage, main, [], AC_MSG_ERROR([Library 'freeimage' is not found]))
			AC_CHECK_HEADERS(FreeImagePlus.h, [], AC_MSG_ERROR([Header 'FreeImagePlus.h' is not found]))
			AC_CHECK_LIB(freeimageplus, main, [], AC_MSG_ERROR([Library 'freeimageplus' is not found]))
			;;
		no)
			;;
		auto)
			MINGLE_IMAGING_FREEIMAGE=yes
			AC_CHECK_HEADERS(FreeImage.h, [], MINGLE_IMAGING_FREEIMAGE=no)
			AC_CHECK_LIB(freeimage, main, [], MINGLE_IMAGING_FREEIMAGE=no)
			AC_CHECK_HEADERS(FreeImagePlus.h, [], MINGLE_IMAGING_FREEIMAGE=no)
			AC_CHECK_LIB(freeimageplus, main, [], MINGLE_IMAGING_FREEIMAGE=no)
			;;
		*)
			AC_MSG_ERROR([Invalid option '$MINGLE_IMAGING_FREEIMAGE'. Use yes, no, or auto])
	esac


	dnl Setting imaging support (DevIL)
	AC_MSG_CHECKING([set imaging support (DevIL)])
	AC_ARG_WITH(imaging_devil,[  --with-imaging-devil=yes|no|auto 
                          set type of devil imaging support (default is auto)],
		MINGLE_IMAGING_DEVIL=$withval
	,
		MINGLE_IMAGING_DEVIL=auto
	)
	AC_MSG_RESULT([$MINGLE_IMAGING_DEVIL])

	dnl Checking headers and libraries (DevIL)
	case $MINGLE_IMAGING_DEVIL in
		yes)
			AC_CHECK_HEADERS(IL/il.h, [], AC_MSG_ERROR([Header 'IL/il.h' is not found]))
			AC_CHECK_LIB(IL, main, [], AC_MSG_ERROR([Library 'IL' is not found]))

			AC_CHECK_HEADERS(IL/ilu.h, [], AC_MSG_ERROR([Header 'IL/ilu.h' is not found]))
			AC_CHECK_LIB(ILU, main, [], AC_MSG_ERROR([Library 'ILU' is not found]))

			AC_CHECK_HEADERS(IL/ilut.h, [], AC_MSG_ERROR([Header 'IL/ilut.h' is not found]))
			AC_CHECK_LIB(ILUT, main, [], AC_MSG_ERROR([Library 'ILUT' is not found]))
			;;
		no)
			;;
		auto)
			MINGLE_IMAGING_DEVIL=yes
			AC_CHECK_HEADERS(IL/il.h, [], MINGLE_IMAGING_DEVIL=no)
			AC_CHECK_LIB(IL, main, [], MINGLE_IMAGING_DEVIL=no)

			AC_CHECK_HEADERS(IL/ilu.h, [], MINGLE_IMAGING_DEVIL=no)
			AC_CHECK_LIB(ILU, main, [], MINGLE_IMAGING_DEVIL=no)

			AC_CHECK_HEADERS(IL/ilut.h, [], MINGLE_IMAGING_DEVIL=no)
			AC_CHECK_LIB(ILUT, main, [], MINGLE_IMAGING_DEVIL=no)
			;;
		*)
			AC_MSG_ERROR([Invalid option '$MINGLE_IMAGING_DEVIL'. Use yes, no, or auto])
	esac


	dnl Setting imaging support (native)
	AC_MSG_CHECKING([set imaging support (native)])
	AC_ARG_WITH(imaging_native,[  --with-imaging-native=yes|no|auto 
                          set type of native imaging support (default is auto)],
		MINGLE_IMAGING_NATIVE=$withval
	,
		MINGLE_IMAGING_NATIVE=auto
	)
	AC_MSG_RESULT([$MINGLE_IMAGING_NATIVE])

	dnl Checking headers and libraries (native)
	case $MINGLE_IMAGING_NATIVE in
		yes)
			dnl No additional library is needed, MinGLE will use internal codecs
			;;
		no)
			;;
		auto)
			dnl MinGLE will be only built with native support if neither of 
			dnl the upper imaging meta libraries present
			MINGLE_IMAGING_NATIVE=yes
			if test "MINGLE_IMAGING_FREEIMAGE" = "yes"; then
				MINGLE_IMAGING_NATIVE=no
			fi
			if test "$MINGLE_IMAGING_DEVIL" = "yes"; then
				MINGLE_IMAGING_NATIVE=no
			fi
			;;
		*)
			AC_MSG_ERROR([Invalid option '$MINGLE_IMAGING_NATIVE'. Use yes, no, or auto])
	esac

	dnl Set results for automake
	AM_CONDITIONAL(MINGLE_IMAGING_FREEIMAGE, test $MINGLE_IMAGING_FREEIMAGE = yes)
	AM_CONDITIONAL(MINGLE_IMAGING_DEVIL, test $MINGLE_IMAGING_DEVIL = yes)
	AM_CONDITIONAL(MINGLE_IMAGING_NATIVE, test $MINGLE_IMAGING_NATIVE = yes)


	dnl Concetenate info string
	MINGLE_IMAGING=""
	if test "$MINGLE_IMAGING_FREEIMAGE" = "yes"; then
		MINGLE_IMAGING="$MINGLE_IMAGING freeimage"
	fi
	if test "$MINGLE_IMAGING_DEVIL" = "yes"; then
		MINGLE_IMAGING="$MINGLE_IMAGING DevIL"
	fi
	if test "$MINGLE_IMAGING_NATIVE" = "yes"; then
		MINGLE_IMAGING="$MINGLE_IMAGING native"
	fi
])

dnl
dnl AC_CHECK_FT2([MINIMUM-VERSION, [ACTION-IF-FOUND [, ACTION-IF-NOT-FOUND]]])
dnl Configure paths for FreeType2
dnl Test for FreeType2, and define FT2_CFLAGS and FT2_LIBS
dnl Marcelo Magallon 2001-10-26, based on gtk.m4 by Owen Taylor
dnl
AC_DEFUN([AC_CHECK_FT2],
[
dnl
dnl Get the cflags and libraries from the freetype-config script
dnl
AC_ARG_WITH(ft-prefix,
[  --with-ft-prefix=PREFIX
                          Prefix where FreeType is installed (optional)],
            ft_config_prefix="$withval", ft_config_prefix="")
AC_ARG_WITH(ft-exec-prefix,
[  --with-ft-exec-prefix=PREFIX
                          Exec prefix where FreeType is installed (optional)],
            ft_config_exec_prefix="$withval", ft_config_exec_prefix="")
AC_ARG_ENABLE(freetypetest,
[  --disable-freetypetest  Do not try to compile and run
                          a test FreeType program],
              [], enable_fttest=yes)

if test x$ft_config_exec_prefix != x ; then
  ft_config_args="$ft_config_args --exec-prefix=$ft_config_exec_prefix"
  if test x${FT2_CONFIG+set} != xset ; then
    FT2_CONFIG=$ft_config_exec_prefix/bin/freetype-config
  fi
fi
if test x$ft_config_prefix != x ; then
  ft_config_args="$ft_config_args --prefix=$ft_config_prefix"
  if test x${FT2_CONFIG+set} != xset ; then
    FT2_CONFIG=$ft_config_prefix/bin/freetype-config
  fi
fi
AC_PATH_PROG(FT2_CONFIG, freetype-config, no)

min_ft_version=ifelse([$1], ,9.1.0,$1)
AC_MSG_CHECKING(for FreeType - version >= $min_ft_version)
no_ft=""
if test "$FT2_CONFIG" = "no" ; then
  no_ft=yes
else
  FT2_CFLAGS=`$FT2_CONFIG $ft_config_args --cflags`
  FT2_LIBS=`$FT2_CONFIG $ft_config_args --libs`
  ft_config_major_version=`$FT2_CONFIG $ft_config_args --version | \
         sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\1/'`
  ft_config_minor_version=`$FT2_CONFIG $ft_config_args --version | \
         sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\2/'`
  ft_config_micro_version=`$FT2_CONFIG $ft_config_args --version | \
         sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\3/'`
  ft_min_major_version=`echo $min_ft_version | \
         sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\1/'`
  ft_min_minor_version=`echo $min_ft_version | \
         sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\2/'`
  ft_min_micro_version=`echo $min_ft_version | \
         sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\3/'`
  if test x$enable_fttest = xyes ; then
    ft_config_is_lt=""
    if test $ft_config_major_version -lt $ft_min_major_version ; then
      ft_config_is_lt=yes
    else
      if test $ft_config_major_version -eq $ft_min_major_version ; then
        if test $ft_config_minor_version -lt $ft_min_minor_version ; then
          ft_config_is_lt=yes
        else
          if test $ft_config_minor_version -eq $ft_min_minor_version ; then
            if test $ft_config_micro_version -lt $ft_min_micro_version ; then
              ft_config_is_lt=yes
            fi
          fi
        fi
      fi
    fi
    if test x$ft_config_is_lt = xyes ; then
      no_ft=yes
    else
      ac_save_CXXFLAGS="$CXXFLAGS"
      ac_save_LIBS="$LIBS"
      CXXFLAGS="$CXXFLAGS $FT2_CFLAGS"
      LIBS="$FT2_LIBS $LIBS"
dnl
dnl Sanity checks for the results of freetype-config to some extent
dnl
      AC_TRY_RUN([
#include <ft2build.h>
#include FT_FREETYPE_H
#include <stdio.h>
#include <stdlib.h>

int
main()
{
  FT_Library library;
  FT_Error error;

  error = FT_Init_FreeType(&library);

  if (error)
    return 1;
  else
  {
    FT_Done_FreeType(library);
    return 0;
  }
}
],, no_ft=yes,[echo $ac_n "cross compiling; assumed OK... $ac_c"])
      CXXFLAGS="$ac_save_CXXFLAGS"
      LIBS="$ac_save_LIBS"
    fi             # test $ft_config_version -lt $ft_min_version
  fi               # test x$enable_fttest = xyes
fi                 # test "$FT2_CONFIG" = "no"
if test x$no_ft = x ; then
   AC_MSG_RESULT(yes)
   ifelse([$2], , :, [$2])
else
   AC_MSG_RESULT(no)
   if test "$FT2_CONFIG" = "no" ; then
     echo "*** The freetype-config script installed by FreeType 2 could not be found."
     echo "*** If FreeType 2 was installed in PREFIX, make sure PREFIX/bin is in"
     echo "*** your path, or set the FT2_CONFIG environment variable to the"
     echo "*** full path to freetype-config."
   else
     if test x$ft_config_is_lt = xyes ; then
       echo "*** Your installed version of the FreeType 2 library is too old."
       echo "*** If you have different versions of FreeType 2, make sure that"
       echo "*** correct values for --with-ft-prefix or --with-ft-exec-prefix"
       echo "*** are used, or set the FT2_CONFIG environment variable to the"
       echo "*** full path to freetype-config."
     else
       echo "*** The FreeType test program failed to run.  If your system uses"
       echo "*** shared libraries and they are installed outside the normal"
       echo "*** system library path, make sure the variable LD_LIBRARY_PATH"
       echo "*** (or whatever is appropiate for your system) is correctly set."
     fi
   fi
   FT2_CFLAGS=""
   FT2_LIBS=""
   ifelse([$3], , :, [$3])
fi
AC_SUBST(FT2_CFLAGS)
AC_SUBST(FT2_LIBS)
])

dnl 
dnl Detect font rendering support on target
dnl 
AC_DEFUN([MINGLE_DETECT_FONT_RENDERING],
[
	dnl Setting font rendering support (basic, atlas based)
	AC_MSG_CHECKING([set font rendering support (basic, atlas based)])
	AC_ARG_WITH(fontrendering_base,[  --with-fontrendering-base=yes|no 
                          set type of basic, atlas based font rendering support (default is yes)],
		MINGLE_FONT_RENDERING_BASE=$withval
	,
		MINGLE_FONT_RENDERING_BASE=yes
	)
	AC_MSG_RESULT([$MINGLE_FONT_RENDERING_BASE])

	dnl Checking headers and libraries (basic, atlas based)
	case $MINGLE_FONT_RENDERING_BASE in
		yes)
			dnl Nothing to check
			;;
		no)
			dnl Nothing to check
			;;
		*)
			AC_MSG_ERROR([Invalid option '$MINGLE_FONT_RENDERING_BASE'. Use yes or no])
	esac

	dnl Setting font rendering support (FreeType2)
	AC_MSG_CHECKING([set font rendering support (FreeType2)])
	AC_ARG_WITH(fontrendering_freetype2,[  --with-fontrendering-freetype2=yes|no|auto 
                          set type of FreeType2 font rendering support (default is auto)],
		MINGLE_FONT_RENDERING_FREETYPE2=$withval
	,
		MINGLE_FONT_RENDERING_FREETYPE2=auto
	)
	AC_MSG_RESULT([$MINGLE_FONT_RENDERING_FREETYPE2])

	dnl Checking headers and libraries (FreeType2)
	case $MINGLE_FONT_RENDERING_FREETYPE2 in
		yes)
			AC_CHECK_FT2(9.0.0, [], AC_MSG_ERROR([FreeType2 version >= 9.0.0 is not found on target]))
			;;
		no)
			;;
		auto)
			AC_CHECK_FT2(9.0.0, [MINGLE_FONT_RENDERING_FREETYPE2=yes], [MINGLE_FONT_RENDERING_FREETYPE2=no])
			;;
		*)
			AC_MSG_ERROR([Invalid option '$MINGLE_FONT_RENDERING_FREETYPE2'. Use yes, no, or auto])
	esac


	dnl Set results for automake
	AM_CONDITIONAL(MINGLE_FONT_RENDERING_BASE, test $MINGLE_FONT_RENDERING_BASE = yes)
	AM_CONDITIONAL(MINGLE_FONT_RENDERING_FREETYPE2, test $MINGLE_FONT_RENDERING_FREETYPE2 = yes)
	if test $MINGLE_FONT_RENDERING_FREETYPE2 = yes; then
		dnl Add to include path and add libraries
		CXXFLAGS="$CXXFLAGS $FT2_CFLAGS"
		LIBS="$FT2_LIBS $LIBS"
	fi


	dnl Concetenate info string
	MINGLE_FONT_RENDERING=""
	if test "$MINGLE_FONT_RENDERING_BASE" = "yes"; then
		MINGLE_FONT_RENDERING="$MINGLE_FONT_RENDERING Base"
	fi
	if test "$MINGLE_FONT_RENDERING_FREETYPE2" = "yes"; then
		MINGLE_FONT_RENDERING="$MINGLE_FONT_RENDERING FreeType2"
	fi
])

dnl 
dnl Detect X Windowing system on target
dnl 
AC_DEFUN([MINGLE_DETECT_XLIB],
[
	AC_ARG_ENABLE(xlib,
		AC_HELP_STRING([--disable-xlib], [Disable X Windowing System detection (default enabled)]),
		[xlib_detection=$enableval],
		[xlib_detection=yes])
	
	AC_MSG_CHECKING([whether to detect X Windowing System])
        case $xlib_detection in
        	yes)
			AC_MSG_RESULT(yes)
			AC_CHECK_HEADERS(X11/Xlib.h, [], [])
			
			dnl When using x86_64 architecture add /usr/X11R6/lib64 to the linker path
			if test "$MINGLE_ARCH" = "x86_64"; then
			    LDFLAGS="$LDFLAGS -L/usr/X11R6/lib64"
			fi
		;;
        	*)
			AC_MSG_RESULT([no, skipped])
        	;;
	esac
])

dnl
dnl Enable/disable library options with native implementation
dnl
AC_DEFUN([MINGLE_ENABLE_NATIVE_OPTIONS],
[
	dnl Frame Buffer Object support
	AC_ARG_ENABLE(fbo,
		AC_HELP_STRING([--disable-fbo], [Disable Frame Buffer Object support (default enabled if GLEW is present)]),
		[MINGLE_FBO=$enableval],
		[MINGLE_FBO=yes])
	if test $MINGLE_GLEW = no; then
	    MINGLE_FBO=no
	fi
	AM_CONDITIONAL(MINGLE_FBO, test $MINGLE_FBO = yes)

	dnl PFS high dynamic range image support
	AC_ARG_ENABLE(pfs,
		AC_HELP_STRING([--disable-pfs], [Disable PFS high dynamic range image support (default enabled)]),
		[MINGLE_PFS=$enableval],
		[MINGLE_PFS=yes])
	AM_CONDITIONAL(MINGLE_PFS, test $MINGLE_PFS = yes)
])

dnl
dnl Enable/disable inbuilt GUI support
dnl
AC_DEFUN([MINGLE_ENABLE_GUI],
[
	AC_ARG_ENABLE(gui,
		AC_HELP_STRING([--disable-gui], [Disable inbuilt mini-GUI (default enabled)]),
		[MINGLE_GUI=$enableval],
		[MINGLE_GUI=yes])
	AM_CONDITIONAL(MINGLE_GUI, test $MINGLE_GUI = yes)
])

dnl
dnl Enable/disable building samples and tools
dnl
AC_DEFUN([MINGLE_ENABLE_ADDITIONAL_BINARIES],
[
	dnl Sample programs
	AC_ARG_ENABLE(samples,
		AC_HELP_STRING([--disable-samples], [Disable building sample programs (default enabled)]),
		[MINGLE_BUILD_SAMPLES=$enableval],
		[MINGLE_BUILD_SAMPLES=yes])
	AM_CONDITIONAL(MINGLE_BUILD_SAMPLES, test $MINGLE_BUILD_SAMPLES = yes)

	dnl Tools
	AC_ARG_ENABLE(tools,
		AC_HELP_STRING([--disable-tools], [Disable building tools (default enabled)]),
		[MINGLE_BUILD_TOOLS=$enableval],
		[MINGLE_BUILD_TOOLS=yes])
	AM_CONDITIONAL(MINGLE_BUILD_TOOLS, test $MINGLE_BUILD_TOOLS = yes)
	
	dnl Define samples absolute install path
	mingle_samplesdir='${abs_top_builddir}/samples/bin'
	AC_SUBST(abs_top_builddir)
	AC_SUBST(mingle_samplesdir)
])

dnl
dnl Dynamically generate include/mingle-gl.h header file
dnl
AC_DEFUN([MINGLE_GENERATE_MINGLE_GL_H],
[
	AC_MSG_NOTICE([generating include/mingle-gl.h])
	cp include/mingle-gl.h-tpl include/mingle-gl.h

	dnl Place GL/glew.h include
	case $MINGLE_GLEW in
		yes)
			sed -e "s|@HOOK-FOR-GLEW@|// Include GL/glew.h first\n#include <GL/glew.h>|" -i include/mingle-gl.h
			;;
		no)
			sed -e "s|@HOOK-FOR-GLEW@|// Note that GLEW is not enabled\n// If it is a problem for you, please recompile MinGLE with enabled GLEW option.|" -i include/mingle-gl.h
			;;
		*)
			AC_MSG_ERROR([Internal error. MINGLE_GLEW='$MINGLE_GLEW' is invalid. Please report this bug.])
	esac
])

