//      ___                       ___           ___           ___       ___     
//     /\__\          ___        /\__\         /\  \         /\__\     /\  \.    
//    /::|  |        /\  \      /::|  |       /::\  \       /:/  /    /::\  \.   
//   /:|:|  |        \:\  \    /:|:|  |      /:/\:\  \     /:/  /    /:/\:\  \.  
//  /:/|:|__|__      /::\__\  /:/|:|  |__   /:/  \:\  \   /:/  /    /::\~\:\  \. 
// /:/ |::::\__\  __/:/\/__/ /:/ |:| /\__\ /:/__/_\:\__\ /:/__/    /:/\:\ \:\__\.
// \/__/~~/:/  / /\/:/  /    \/__|:|/:/  / \:\  /\ \/__/ \:\  \    \:\~\:\ \/__/
//       /:/  /  \::/__/         |:/:/  /   \:\ \:\__\    \:\  \    \:\ \:\__\.  
//      /:/  /    \:\__\         |::/  /     \:\/:/  /     \:\  \    \:\ \/__/  
//     /:/  /      \/__/         /:/  /       \::/  /       \:\__\    \:\__\.    
//     \/__/                     \/__/         \/__/         \/__/     \/__/    
// 
// =============================================================================
//                       Minimalist OpenGL Environment
// =============================================================================
//
// This file is part of Minimalist OpenGL Environment (MinGLE)
//
// Version: 0.2.0
// Author: Balazs Domonkos
// Filename: include/DynLoad.h
// Creation Date: December 12th 2007
// Revision Date: December 12th 2007
//
//
// The Minimalist OpenGL Environment is free software: you can
// redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// The Minimalist OpenGL Environment is distributed in the hope
// that it will be useful, but WITHOUT ANY WARRANTY; without
// even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License
// along with Minimalist OpenGL Environment.  If not, see 
// <http://www.gnu.org/licenses/>. See the COPYING file included 
// with this distribution file for license details.
//

/// @file DynLoad.h
/// Dynamic load library support

#ifndef __MINGLE_DYNLOAD_H__
#define __MINGLE_DYNLOAD_H__

// Include class definitions
#include "classes.h"

// MinGLE headers
#include "Exception.h"

namespace MinGLE {

// =============================================================================

class DynLoad {

public:
	enum LoadMode {
		LAZY,
		IMMEDIATE
	};

	static const LoadMode DEFAULT_LOAD_MODE = LAZY;
	static const std::string DYNAMIC_LIBRARY_EXTENSION;
	static const unsigned DYNAMIC_LIBRARY_TYPE_MASK;

protected:
	void *mHandle;

public:
	DynLoad(const std::string& libraryName,
			LoadMode loadMode = DEFAULT_LOAD_MODE);
	~DynLoad();

	bool hasSymbol(const std::string& symbol);

	template<typename T> T getSymbol(const std::string& symbol);

protected:
	void _load(const std::string& libraryName, LoadMode loadMode);
	void _unload();
	void *_getSymbol(const std::string& symbol);

}; // class DynLoad

template<typename T> T DynLoad::getSymbol(const std::string& symbol) {
	Assert(mHandle, "DynLoad::getSymbol");

	// This is an issue: "When standards collide: the problem with dlsym"
	// see topic http://www.trilithium.com/johan/2004/12/problem-with-dlsym/
	// and http://groups.google.pl/group/comp.lang.c++.moderated/browse_frm/thread/51544112db2a6f1/

	// Note that unsigned long should be large enough to store the address
	reinterpret_cast<T>(reinterpret_cast<unsigned long>(_getSymbol(symbol))); 
	// return reinterpret_cast<T>(_getSymbol(symbol));
}

// =============================================================================

class DynLib : public DynLoad {

public:
	typedef void (*DynLibLoaderFunction)();
	typedef void (*DynLibUnLoaderFunction)();

	static const std::string DYNAMIC_LIBRARY_LOADER_FUNCTION;
	static const std::string DYNAMIC_LIBRARY_UNLOADER_FUNCTION;

public:
			DynLib(const std::string& libraryName,
					LoadMode loadMode = DEFAULT_LOAD_MODE);
	~DynLib();

}; // class DynLib

} // namespace MinGLE

#endif // __MINGLE_DYNLOAD_H__
