//      ___                       ___           ___           ___       ___     
//     /\__\          ___        /\__\         /\  \         /\__\     /\  \.    
//    /::|  |        /\  \      /::|  |       /::\  \       /:/  /    /::\  \.   
//   /:|:|  |        \:\  \    /:|:|  |      /:/\:\  \     /:/  /    /:/\:\  \.  
//  /:/|:|__|__      /::\__\  /:/|:|  |__   /:/  \:\  \   /:/  /    /::\~\:\  \. 
// /:/ |::::\__\  __/:/\/__/ /:/ |:| /\__\ /:/__/_\:\__\ /:/__/    /:/\:\ \:\__\.
// \/__/~~/:/  / /\/:/  /    \/__|:|/:/  / \:\  /\ \/__/ \:\  \    \:\~\:\ \/__/
//       /:/  /  \::/__/         |:/:/  /   \:\ \:\__\    \:\  \    \:\ \:\__\.  
//      /:/  /    \:\__\         |::/  /     \:\/:/  /     \:\  \    \:\ \/__/  
//     /:/  /      \/__/         /:/  /       \::/  /       \:\__\    \:\__\.    
//     \/__/                     \/__/         \/__/         \/__/     \/__/    
// 
// =============================================================================
//                       Minimalist OpenGL Environment
// =============================================================================
//
// This file is part of Minimalist OpenGL Environment (MinGLE)
//
// Version: 0.2.0
// Author: Balazs Domonkos
// Filename: include/Exception.h
// Creation Date: December 12th 2007
// Revision Date: December 12th 2007
//
//
// The Minimalist OpenGL Environment is free software: you can
// redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// The Minimalist OpenGL Environment is distributed in the hope
// that it will be useful, but WITHOUT ANY WARRANTY; without
// even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License
// along with Minimalist OpenGL Environment.  If not, see 
// <http://www.gnu.org/licenses/>. See the COPYING file included 
// with this distribution file for license details.
//

/// @file Exception.h
/// Exception class

#ifndef __MINGLE_EXCEPTION_H__
#define __MINGLE_EXCEPTION_H__

// Include class definitions
#include "classes.h"

// System headers
#include <string>

// TODO use  __PRETTY_FUNCTION__ or __FUNCTION__ instead

///> Macro for throwing exception.
#  define Except(func, desc) throw(MinGLE::Exception(desc, __FILE__, func, __LINE__ ))
///> Macro for asserting.
#  define Assert(cond, func) if(!(cond)) Except(func, "Assertion error: " #cond)

namespace MinGLE {

class Exception {
protected:
	static const std::string UNKNOWN_DESCRIPTION;
	static const std::string UNKNOWN_FILE;
	static const std::string UNKNOWN_METHOD;
	static const int UNKNOWN_LINE;

public:
	std::string mDescription;
	std::string mFile;
	std::string mMethod;
	int mLine;

public:
	inline Exception(const std::string& description = UNKNOWN_DESCRIPTION,
			const std::string& file = UNKNOWN_FILE,
			const std::string& method = UNKNOWN_METHOD,
			const int& line = UNKNOWN_LINE) :
		mDescription(description), mFile(file), mMethod(method), mLine(line) {
	}
	inline virtual ~Exception() {
	}
}; // class Exception

} // namespace MinGLE

#endif // __MINGLE_EXCEPTION_H__
