//      ___                       ___           ___           ___       ___     
//     /\__\          ___        /\__\         /\  \         /\__\     /\  \.    
//    /::|  |        /\  \      /::|  |       /::\  \       /:/  /    /::\  \.   
//   /:|:|  |        \:\  \    /:|:|  |      /:/\:\  \     /:/  /    /:/\:\  \.  
//  /:/|:|__|__      /::\__\  /:/|:|  |__   /:/  \:\  \   /:/  /    /::\~\:\  \. 
// /:/ |::::\__\  __/:/\/__/ /:/ |:| /\__\ /:/__/_\:\__\ /:/__/    /:/\:\ \:\__\.
// \/__/~~/:/  / /\/:/  /    \/__|:|/:/  / \:\  /\ \/__/ \:\  \    \:\~\:\ \/__/
//       /:/  /  \::/__/         |:/:/  /   \:\ \:\__\    \:\  \    \:\ \:\__\.  
//      /:/  /    \:\__\         |::/  /     \:\/:/  /     \:\  \    \:\ \/__/  
//     /:/  /      \/__/         /:/  /       \::/  /       \:\__\    \:\__\.    
//     \/__/                     \/__/         \/__/         \/__/     \/__/    
// 
// =============================================================================
//                       Minimalist OpenGL Environment
// =============================================================================
//
// This file is part of Minimalist OpenGL Environment (MinGLE)
//
// Version: 0.2.0
// Author: Balazs Domonkos
// Filename: include/GLUtils.h
// Creation Date: December 12th 2007
// Revision Date: December 12th 2007
//
//
// The Minimalist OpenGL Environment is free software: you can
// redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// The Minimalist OpenGL Environment is distributed in the hope
// that it will be useful, but WITHOUT ANY WARRANTY; without
// even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License
// along with Minimalist OpenGL Environment.  If not, see 
// <http://www.gnu.org/licenses/>. See the COPYING file included 
// with this distribution file for license details.
//

/// @file GLUtils.h
/// OpenGL utils

#ifndef __MINGLE_GLUTILS_H__
#define __MINGLE_GLUTILS_H__

// Include class definitions
#include "classes.h"
#include "mingle-gl.h"

namespace MinGLE {

class GLUtils {
public:
	static unsigned calculateBytesPerPixel(GLenum format, GLenum type);
	static unsigned calculateNumberOfChannels(GLenum format);
	static GLenum getTextureType(GLenum target, GLint level = 0,
			bool forceStandard = false);
	static GLenum convertToStandardType(GLenum type);

	static double calculateDepth(const double *projectionModelViewMatrix,
			const double *position);
	static double calculateDepth(const double *position);

	static void getModelViewMatrix(double *modelViewMatrix,
			bool transpose = true);
	static void setTranslation(double *modelViewMatrix, const double x,
			const double y, const double z, bool transpose = true);
	static void getProjectionModelViewMatrix(double *projectionModelViewMatrix,
			bool transpose = true);

	static void getViewDirection(double &x, double &y, double &z);

	/// Retrieve the point of the eye in object space
	static void getViewPoint(double &x, double &y, double &z);

	// Transform a point from object space to eye space
	static void modelTransformation(const double *matrix, double &x, double &y,
			double &z);

	static unsigned getMax3DTextureSize();

	static void drawGears(GLfloat angle = 0.0f);

protected:
	static void _drawGear(const GLfloat innerRadius, const GLfloat outerRadius,
			const GLfloat width, const GLint teeth, const GLfloat toothDepth);
};
// class GLUtils

} // namespace MinGLE

#endif // __MINGLE_GLUTILS_H__
