//      ___                       ___           ___           ___       ___     
//     /\__\          ___        /\__\         /\  \         /\__\     /\  \.    
//    /::|  |        /\  \      /::|  |       /::\  \       /:/  /    /::\  \.   
//   /:|:|  |        \:\  \    /:|:|  |      /:/\:\  \     /:/  /    /:/\:\  \.  
//  /:/|:|__|__      /::\__\  /:/|:|  |__   /:/  \:\  \   /:/  /    /::\~\:\  \. 
// /:/ |::::\__\  __/:/\/__/ /:/ |:| /\__\ /:/__/_\:\__\ /:/__/    /:/\:\ \:\__\.
// \/__/~~/:/  / /\/:/  /    \/__|:|/:/  / \:\  /\ \/__/ \:\  \    \:\~\:\ \/__/
//       /:/  /  \::/__/         |:/:/  /   \:\ \:\__\    \:\  \    \:\ \:\__\.  
//      /:/  /    \:\__\         |::/  /     \:\/:/  /     \:\  \    \:\ \/__/  
//     /:/  /      \/__/         /:/  /       \::/  /       \:\__\    \:\__\.    
//     \/__/                     \/__/         \/__/         \/__/     \/__/    
// 
// =============================================================================
//                       Minimalist OpenGL Environment
// =============================================================================
//
// This file is part of Minimalist OpenGL Environment (MinGLE)
//
// Version: 0.2.0
// Author: Balazs Domonkos
// Filename: include/Serialization.h
// Creation Date: January 27th 2008
// Revision Date: January 27th 2008
//
//
// The Minimalist OpenGL Environment is free software: you can
// redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// The Minimalist OpenGL Environment is distributed in the hope
// that it will be useful, but WITHOUT ANY WARRANTY; without
// even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License
// along with Minimalist OpenGL Environment.  If not, see 
// <http://www.gnu.org/licenses/>. See the COPYING file included 
// with this distribution file for license details.
//

/// @file Serialization.h
/// Serialization class

#ifndef __MINGLE_SERIALIZATION_H__
#define __MINGLE_SERIALIZATION_H__

// Include class definitions
#include "classes.h"

// MinGLE headers
#include "Configuration.h"

namespace MinGLE {

template <typename SERIALIZED_T, typename OBJECT_T> class Serialization {
public:
	static SERIALIZED_T *serialize(OBJECT_T *object, SERIALIZED_T *serialData = 0);
	static OBJECT_T *deserialize(SERIALIZED_T *serialData, OBJECT_T *object = 0);
}; // class Serialization

// =============================================================================

template <typename OBJECT_T> class EntrySerialization :
	public Serialization <Configuration::Entry, OBJECT_T> {
};

#define EntrySerialization_Declaration(OBJECT_T) \
template <> class EntrySerialization<OBJECT_T> { \
public: \
	static Configuration::Entry *serialize(OBJECT_T *object, Configuration::Entry *serialData = 0); \
	static OBJECT_T *deserialize(Configuration::Entry *serialData, OBJECT_T *object = 0); \
}

} // namespace MinGLE

#endif // __MINGLE_SERIALIZATION_H__
