//      ___                       ___           ___           ___       ___     
//     /\__\          ___        /\__\         /\  \         /\__\     /\  \.    
//    /::|  |        /\  \      /::|  |       /::\  \       /:/  /    /::\  \.   
//   /:|:|  |        \:\  \    /:|:|  |      /:/\:\  \     /:/  /    /:/\:\  \.  
//  /:/|:|__|__      /::\__\  /:/|:|  |__   /:/  \:\  \   /:/  /    /::\~\:\  \. 
// /:/ |::::\__\  __/:/\/__/ /:/ |:| /\__\ /:/__/_\:\__\ /:/__/    /:/\:\ \:\__\.
// \/__/~~/:/  / /\/:/  /    \/__|:|/:/  / \:\  /\ \/__/ \:\  \    \:\~\:\ \/__/
//       /:/  /  \::/__/         |:/:/  /   \:\ \:\__\    \:\  \    \:\ \:\__\.  
//      /:/  /    \:\__\         |::/  /     \:\/:/  /     \:\  \    \:\ \/__/  
//     /:/  /      \/__/         /:/  /       \::/  /       \:\__\    \:\__\.    
//     \/__/                     \/__/         \/__/         \/__/     \/__/    
// 
// =============================================================================
//                       Minimalist OpenGL Environment
// =============================================================================
//
// This file is part of Minimalist OpenGL Environment (MinGLE)
//
// Version: 0.2.0
// Author: Balazs Domonkos
// Filename: include/StringUtils.h
// Creation Date: December 12th 2007
// Revision Date: December 12th 2007
//
//
// The Minimalist OpenGL Environment is free software: you can
// redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// The Minimalist OpenGL Environment is distributed in the hope
// that it will be useful, but WITHOUT ANY WARRANTY; without
// even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License
// along with Minimalist OpenGL Environment.  If not, see 
// <http://www.gnu.org/licenses/>. See the COPYING file included 
// with this distribution file for license details.
//

/// @file StringUtils.h
/// String handling utils

#ifndef __MINGLE_STRINGUTILS_H__
#define __MINGLE_STRINGUTILS_H__

// Include class definitions
#include "classes.h"

// MinGLE headers
#include <Exception.h>

// System headers
#include <string>
#include <sstream>
#include <vector>

namespace MinGLE {

class StringUtils {

public:
	static const std::string DEFAULT_TRIM_DELIMITERS;
	static const int DEFAULT_FLOATING_POINT_PRECISION;

public:
	static void trim(std::string& str,
			const std::string& delimiters = DEFAULT_TRIM_DELIMITERS);

	static std::string toString(const double value,
			const int precision = DEFAULT_FLOATING_POINT_PRECISION);

	static unsigned parseUnsigned(const std::string& str);
	static double parseDouble(const std::string& str);
	static float parseFloat(const std::string& str);

	// Generic toString methods
	template<typename T> inline static void toString(std::string& str,
			const T& value) {
		std::ostringstream osstr;
		osstr << value;
		str = osstr.str();
	}
	template<typename T> inline static std::string toString(const T& value) {
		std::string str;
		toString(str, value);
		return str;
	}

	// Generic parse methods
	template<typename T> inline static void parse(T& value,
			const std::string& str) {
		std::istringstream isstr(str);
		Assert(isstr >> value, "StringUtils::parse");
	}
	template<typename T> inline static T parse(const std::string& str) {
		T value;
		parse(value, str);
		return value;
	}

	static void tokenize(const std::string& str,
			std::vector<std::string>& tokens,
			const std::string& tokenDelimiters = " ",
			const bool needsTrim = false,
			const std::string& trimDelimiters = DEFAULT_TRIM_DELIMITERS);

	static std::string subStringBefore(const std::string& str, const std::string& pattern);
	static std::string subStringAfter(const std::string& str, const std::string& pattern);
	
}; // class StringUtils

// Explicit overload for boolean type
template<>
inline void StringUtils::parse(bool& value, const std::string& str) {
	if(str == "true") value = true; 
	else if(str == "false") value = false;
	else Except("StringUtils::parse", "Invalid logical value `" + str + '\"');
}

} // namespace MinGLE

#endif // __MINGLE_STRINGUTILS_H__
