//      ___                       ___           ___           ___       ___     
//     /\__\          ___        /\__\         /\  \         /\__\     /\  \.    
//    /::|  |        /\  \      /::|  |       /::\  \       /:/  /    /::\  \.   
//   /:|:|  |        \:\  \    /:|:|  |      /:/\:\  \     /:/  /    /:/\:\  \.  
//  /:/|:|__|__      /::\__\  /:/|:|  |__   /:/  \:\  \   /:/  /    /::\~\:\  \. 
// /:/ |::::\__\  __/:/\/__/ /:/ |:| /\__\ /:/__/_\:\__\ /:/__/    /:/\:\ \:\__\.
// \/__/~~/:/  / /\/:/  /    \/__|:|/:/  / \:\  /\ \/__/ \:\  \    \:\~\:\ \/__/
//       /:/  /  \::/__/         |:/:/  /   \:\ \:\__\    \:\  \    \:\ \:\__\.  
//      /:/  /    \:\__\         |::/  /     \:\/:/  /     \:\  \    \:\ \/__/  
//     /:/  /      \/__/         /:/  /       \::/  /       \:\__\    \:\__\.    
//     \/__/                     \/__/         \/__/         \/__/     \/__/    
// 
// =============================================================================
//                       Minimalist OpenGL Environment
// =============================================================================
//
// This file is part of Minimalist OpenGL Environment (MinGLE)
//
// Version: 0.2.0
// Author: Balazs Domonkos
// Filename: include/Timer.h
// Creation Date: December 12th 2007
// Revision Date: December 12th 2007
//
//
// The Minimalist OpenGL Environment is free software: you can
// redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// The Minimalist OpenGL Environment is distributed in the hope
// that it will be useful, but WITHOUT ANY WARRANTY; without
// even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License
// along with Minimalist OpenGL Environment.  If not, see 
// <http://www.gnu.org/licenses/>. See the COPYING file included 
// with this distribution file for license details.
//

/// @file Timer.h
/// Generic timer class

#ifndef __MINGLE_TIMER_H__
#define __MINGLE_TIMER_H__

// Include class definitions
#include "classes.h"

namespace MinGLE {

class Timer {
public:
	enum TimerType {TIMER_ONE_SHOT, TIMER_PERIODIC};
	static const bool DEFAULT_START_ON_REGISTRATION = true;
	static const bool DEFAULT_ADAPT_TO_DESIRED_FPS = true;

protected:
	static const double DEFAULT_GRANULARITY = 0.0;

protected:
	double mPeriod;
	TimerType mType;
	double mLastTickTime;
	double mElapsedTime;
	bool mRunning;

	Window *mWindow;
	TimerListener *mTimerListener;

	// Features
	bool mAutoStartOnRegistration;
	bool mAdaptToDesiredFPS;

public:
	Timer(Window *window, double period, TimerType type = TIMER_PERIODIC);
	virtual ~Timer();

	// Static methods
	static double getSystemTime();

	// Start and stop
	virtual void start();
	virtual void pause();
	virtual void stop();

	// Getters
	inline virtual TimerType getType() {
		return mType;
	}
	inline virtual double getGranularity() {
		return DEFAULT_GRANULARITY;
	}
	inline virtual double getLastTickTime() {
		return mLastTickTime;
	}
	inline virtual double getElapsedTime() {
		return mElapsedTime;
	}
	inline virtual bool isRunning() {
		return mRunning;
	}
	inline virtual bool getAutoStartOnRegistration() {
		return mAutoStartOnRegistration;
	}
	inline virtual bool getAdaptToDesiredFPS() {
		return mAdaptToDesiredFPS;
	}

	// Setters
	inline virtual void setAutoStartOnRegistration(bool flag) {
		mAutoStartOnRegistration = flag;
	}
	inline virtual void setAdaptToDesiredFPS(bool flag) {
		mAdaptToDesiredFPS = flag;
	}

	// Register timer
	virtual void registerTimerListener(TimerListener *timerListener);
	virtual TimerListener *removeTimerListener();
}; // class Timer

} // namespace MinGLE

#endif // __MINGLE_TIMER_H__
