//      ___                       ___           ___           ___       ___     
//     /\__\          ___        /\__\         /\  \         /\__\     /\  \.    
//    /::|  |        /\  \      /::|  |       /::\  \       /:/  /    /::\  \.   
//   /:|:|  |        \:\  \    /:|:|  |      /:/\:\  \     /:/  /    /:/\:\  \.  
//  /:/|:|__|__      /::\__\  /:/|:|  |__   /:/  \:\  \   /:/  /    /::\~\:\  \. 
// /:/ |::::\__\  __/:/\/__/ /:/ |:| /\__\ /:/__/_\:\__\ /:/__/    /:/\:\ \:\__\.
// \/__/~~/:/  / /\/:/  /    \/__|:|/:/  / \:\  /\ \/__/ \:\  \    \:\~\:\ \/__/
//       /:/  /  \::/__/         |:/:/  /   \:\ \:\__\    \:\  \    \:\ \:\__\.  
//      /:/  /    \:\__\         |::/  /     \:\/:/  /     \:\  \    \:\ \/__/  
//     /:/  /      \/__/         /:/  /       \::/  /       \:\__\    \:\__\.    
//     \/__/                     \/__/         \/__/         \/__/     \/__/    
// 
// =============================================================================
//                       Minimalist OpenGL Environment
// =============================================================================
//
// This file is part of Minimalist OpenGL Environment (MinGLE)
//
// Version: 0.2.0
// Author: Balazs Domonkos
// Filename: include/WindowListener.h
// Creation Date: December 12th 2007
// Revision Date: December 12th 2007
//
//
// The Minimalist OpenGL Environment is free software: you can
// redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// The Minimalist OpenGL Environment is distributed in the hope
// that it will be useful, but WITHOUT ANY WARRANTY; without
// even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License
// along with Minimalist OpenGL Environment.  If not, see 
// <http://www.gnu.org/licenses/>. See the COPYING file included 
// with this distribution file for license details.
//

/// @file WindowListener.h
/// Rendering window event listener class

#ifndef __MINGLE_WINDOWLISTENER_H__
#define __MINGLE_WINDOWLISTENER_H__

// Include class definitions
#include "classes.h"

// MinGLE headers
#include "InputListener.h"

namespace MinGLE {
class WindowListener : public InputAdapter {
	friend class Window;
	friend class GLUTWindow;

public:
	enum WindowEventType {
		RenderEvent = InputListener::RegisterMouseWheelEvent+1,
		IdleEvent, ResizeEvent, RepositionEvent, ShowEvent, HideEvent
	};

	struct WindowEvent : public InputEvent {
		unsigned width, height;
		int left, top;
	};

protected:
	Window *mWindow;
	bool mNeedRedisplay;

public:
	WindowListener();
	virtual ~WindowListener();

protected:
	virtual inline bool onRender() {
		return false;
	}
	virtual inline bool onIdle() {
		return false;
	}

	virtual inline bool onResize(unsigned width, unsigned height) {
		return false;
	}
	virtual inline bool onReposition(int left, int top) {
		return false;
	}

	virtual inline bool onShow() {
		return false;
	}
	virtual inline bool onHide() {
		return false;
	}

	virtual inline void redisplayRequest() {
		mNeedRedisplay = true;
	}
}; // class WindowListener

} // namespace MinGLE

#endif // __MINGLE_WINDOWLISTENER_H__
