//      ___                       ___           ___           ___       ___     
//     /\__\          ___        /\__\         /\  \         /\__\     /\  \.    
//    /::|  |        /\  \      /::|  |       /::\  \       /:/  /    /::\  \.   
//   /:|:|  |        \:\  \    /:|:|  |      /:/\:\  \     /:/  /    /:/\:\  \.  
//  /:/|:|__|__      /::\__\  /:/|:|  |__   /:/  \:\  \   /:/  /    /::\~\:\  \. 
// /:/ |::::\__\  __/:/\/__/ /:/ |:| /\__\ /:/__/_\:\__\ /:/__/    /:/\:\ \:\__\.
// \/__/~~/:/  / /\/:/  /    \/__|:|/:/  / \:\  /\ \/__/ \:\  \    \:\~\:\ \/__/
//       /:/  /  \::/__/         |:/:/  /   \:\ \:\__\    \:\  \    \:\ \:\__\.  
//      /:/  /    \:\__\         |::/  /     \:\/:/  /     \:\  \    \:\ \/__/  
//     /:/  /      \/__/         /:/  /       \::/  /       \:\__\    \:\__\.    
//     \/__/                     \/__/         \/__/         \/__/     \/__/    
// 
// =============================================================================
//                       Minimalist OpenGL Environment
// =============================================================================
//
// This file is part of Minimalist OpenGL Environment (MinGLE)
//
// Version: 0.2.0
// Author: Balazs Domonkos
// Filename: include/classes.h
// Creation Date: November 28th 2007
// Revision Date: November 28th 2007
//
//
// The Minimalist OpenGL Environment is free software: you can
// redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// The Minimalist OpenGL Environment is distributed in the hope
// that it will be useful, but WITHOUT ANY WARRANTY; without
// even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License
// along with Minimalist OpenGL Environment.  If not, see 
// <http://www.gnu.org/licenses/>. See the COPYING file included 
// with this distribution file for license details.
//

/// @file classes.h
/// Class definitions

#ifndef __MINGLE_CLASSES_H__
#define __MINGLE_CLASSES_H__

namespace MinGLE {

//
// Classes for basic funcionality
//

class Exception;
template<typename T> class Manager;
template<typename T, typename Functionality> class ProviderManager;
class DynLoad;
class DynLib;
class Listener;

//
// Classes for rendering systems
//
class System;
class SystemListener;

//
// Classes for configuration file support
//
class Configuration;
class ConfigurationManager;

//
// Classes for serialization support
//
template <typename SERIALIZED_T, typename OBJECT_T> class Serialization;
template <typename OBJECT_T> class EntrySerialization;

//
// Basic utility classes
//

class MathUtils;
class GLUtils;
class StringUtils;

//
// Classes for rendering window
//

class Window;
class WindowProxy;
class InputListener;
class InputAdapter;
class WindowListener;

//
// Classes for time handling
//

class Timer;
class TimerListener;

//
// Classes for file handling
//

class VirtualFileSystem;
class VirtualFileSystemManager;

//
// Classes for threading
//
class Mutex;
class Runnable;
class Thread;
class Threading;
template<typename T> class GuardedObject;
template<typename T> class GuardedPointer;
//class Process;

//
// Classes for image handling
//
class Image;
class Imaging;
template<typename INDEX, typename COORDT> class Atlas;
template<typename INDEX> class ImageAtlas;
template<typename INDEX> class TextureAtlas;

//
// Classes for font handling
//

class Font;

//
// Classes for navigation based user interface
//

class Camera;
class Navigator;
class Animator;

//
// Headers for widget based user interface
//

class Widget;
class Panel;
class GuiSheet;
class TextField;
class CheckBox;
class Slider;

} // namespace MinGLE

#endif // __MINGLE_CLASSES_H__
