//      ___                       ___           ___           ___       ___     
//     /\__\          ___        /\__\         /\  \         /\__\     /\  \.    
//    /::|  |        /\  \      /::|  |       /::\  \       /:/  /    /::\  \.   
//   /:|:|  |        \:\  \    /:|:|  |      /:/\:\  \     /:/  /    /:/\:\  \.  
//  /:/|:|__|__      /::\__\  /:/|:|  |__   /:/  \:\  \   /:/  /    /::\~\:\  \. 
// /:/ |::::\__\  __/:/\/__/ /:/ |:| /\__\ /:/__/_\:\__\ /:/__/    /:/\:\ \:\__\.
// \/__/~~/:/  / /\/:/  /    \/__|:|/:/  / \:\  /\ \/__/ \:\  \    \:\~\:\ \/__/
//       /:/  /  \::/__/         |:/:/  /   \:\ \:\__\    \:\  \    \:\ \:\__\.  
//      /:/  /    \:\__\         |::/  /     \:\/:/  /     \:\  \    \:\ \/__/  
//     /:/  /      \/__/         /:/  /       \::/  /       \:\__\    \:\__\.    
//     \/__/                     \/__/         \/__/         \/__/     \/__/    
// 
// =============================================================================
//                       Minimalist OpenGL Environment
// =============================================================================
//
// This file is part of Minimalist OpenGL Environment (MinGLE)
//
// Version: 0.2.0
// Author: Balazs Domonkos
// Filename: src/Cameras.cpp
// Creation Date: December 12th 2007
// Revision Date: December 12th 2007
//
//
// The Minimalist OpenGL Environment is free software: you can
// redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// The Minimalist OpenGL Environment is distributed in the hope
// that it will be useful, but WITHOUT ANY WARRANTY; without
// even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License
// along with Minimalist OpenGL Environment.  If not, see 
// <http://www.gnu.org/licenses/>. See the COPYING file included 
// with this distribution file for license details.
//

/// @file Cameras.cpp
/// Camera classes

// Header that contains the declaration
#include <Cameras.h>

// MinGLE headers
#include <mingle-gl.h>
#include <string.h>

namespace MinGLE {

const double Camera::DEFAULT_CAMERA_POSITION[3] = {1.0, 1.0, 1.0};
const double Camera::DEFAULT_CAMERA_TARGET[3] = {0.0, 0.0, 0.0};
const double Camera::DEFAULT_UP_VECTOR[3] = {0.0, 1.0, 0.0};

Camera::Camera() :
	mNearClip(DEFAULT_NEAR_CLIP), mFarClip(DEFAULT_FAR_CLIP),
	mAutoRecalculate(DEFAULT_AUTORECALCULATE)
{
	memcpy(mPosition, DEFAULT_CAMERA_POSITION, 3*sizeof(double));
	memcpy(mTarget, DEFAULT_CAMERA_TARGET, 3*sizeof(double));
	memcpy(mUpVector, DEFAULT_UP_VECTOR, 3*sizeof(double));
}

Camera::~Camera() {
}

// =============================================================================

GLManagedCamera::~GLManagedCamera() {
}

void GLManagedCamera::getProjectionMatrix(double *matrix) {
	recalculateProjectionMatrix();
	glGetDoublev(GL_PROJECTION_MATRIX, matrix);
}

void GLManagedCamera::getModelViewMatrix(double *matrix) {
	recalculateModelViewMatrix();
	glGetDoublev(GL_MODELVIEW_MATRIX, matrix);
}

// =============================================================================

SymmetricFrustumCamera::SymmetricFrustumCamera(double fovy) :
	mFovY(fovy)
{
}

SymmetricFrustumCamera::~SymmetricFrustumCamera() {
}

void SymmetricFrustumCamera::recalculateProjectionMatrix() {
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(mFovY, mImageWidth / mImageHeight, mNearClip, mFarClip);
}
    		
void SymmetricFrustumCamera::recalculateModelViewMatrix() {
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	gluLookAt(mPosition[0], mPosition[1], mPosition[2], 
			  mTarget[0], mTarget[1], mTarget[2], 
			  mUpVector[0], mUpVector[1], mUpVector[2]);
}    		

} // namespace MinGLE
