//      ___                       ___           ___           ___       ___     
//     /\__\          ___        /\__\         /\  \         /\__\     /\  \.    
//    /::|  |        /\  \      /::|  |       /::\  \       /:/  /    /::\  \.   
//   /:|:|  |        \:\  \    /:|:|  |      /:/\:\  \     /:/  /    /:/\:\  \.  
//  /:/|:|__|__      /::\__\  /:/|:|  |__   /:/  \:\  \   /:/  /    /::\~\:\  \. 
// /:/ |::::\__\  __/:/\/__/ /:/ |:| /\__\ /:/__/_\:\__\ /:/__/    /:/\:\ \:\__\.
// \/__/~~/:/  / /\/:/  /    \/__|:|/:/  / \:\  /\ \/__/ \:\  \    \:\~\:\ \/__/
//       /:/  /  \::/__/         |:/:/  /   \:\ \:\__\    \:\  \    \:\ \:\__\.  
//      /:/  /    \:\__\         |::/  /     \:\/:/  /     \:\  \    \:\ \/__/  
//     /:/  /      \/__/         /:/  /       \::/  /       \:\__\    \:\__\.    
//     \/__/                     \/__/         \/__/         \/__/     \/__/    
// 
// =============================================================================
//                       Minimalist OpenGL Environment
// =============================================================================
//
// This file is part of Minimalist OpenGL Environment (MinGLE)
//
// Version: 0.2.0
// Author: Balazs Domonkos
// Filename: src/InputListener.cpp
// Creation Date: December 12th 2007
// Revision Date: December 12th 2007
//
//
// The Minimalist OpenGL Environment is free software: you can
// redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// The Minimalist OpenGL Environment is distributed in the hope
// that it will be useful, but WITHOUT ANY WARRANTY; without
// even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License
// along with Minimalist OpenGL Environment.  If not, see 
// <http://www.gnu.org/licenses/>. See the COPYING file included 
// with this distribution file for license details.
//

/// @file InputListener.cpp
/// Input device listener class

// Header that contains the declaration
#include <InputListener.h>

// System headers
#include <stdio.h>

namespace MinGLE {

InputListener::InputListener() {
}

InputListener::~InputListener() {
}

// Protected event hander methods (cannot be called explicitly)
bool InputListener::onKeyEvent(int key, unsigned modifiers, KeyEventState state) {

	/*
	switch(state) {
		case KEY_DOWN: printf("keydown "); break;
		case KEY_UP: printf("keyup "); break;
		case KEY_PRESS: printf("keypress "); break;
	}
	
	printf("%c (%03X) ", key, key);
	
	printf("[%c][%c][%c][%c][%c][%c]\n", modifiers&KEY_LEFT_SHIFT?'S':' ', modifiers&KEY_LEFT_CTRL?'C':' ', 
		modifiers&KEY_LEFT_ALT?'A':' ', modifiers&KEY_RIGHT_ALT?'A':' ', modifiers&KEY_RIGHT_CTRL?'C':' ', 
		modifiers&KEY_RIGHT_SHIFT?'S':' ');
	*/

	// False means the event has not been handled
	return false;
}

bool InputListener::onMouseButtonEvent(int x, int y, MouseButton button, MouseEventState state) {

	/*	
	printf("button #%d ", button);
	switch(state) {
		case BUTTON_DOWN: printf("down\n"); break; 
		case BUTTON_UP: printf("up\n"); break; 
		case BUTTON_CLICK: printf("click\n"); break; 
		case BUTTON_DBLCLICK: printf("dblclick\n"); break; 
	}
	*/

	// False means the event has not been handled
	return false;
}

bool InputListener::onMouseMoveEvent(int x, int y, MouseMoveState state) {
	/*
	switch(state) {
		case ENTER: printf("enter\n"); break;
		case HOVER: printf("x: %4d y: %4d\n", x, y); break;
		case LEAVE: printf("leave\n"); break;
	}
	*/
	
	// False means the event has not been handled
	return false;
}

bool InputListener::onMouseWheelEvent(int x, int y, int delta) {
	//printf("wh: %d\n", delta);

	// False means the event has not been handled
	return false;
}

// =============================================================================

InputAdapter::InputAdapter() :
	mKeyboardModifiers(0), 
	mMouseX(0), mMouseY(0), 
	mMouseLastX(0), mMouseLastY(0), 
	mMouseDownX(0), mMouseDownY(0),
	mMouseWheelPos(0) 
{
	// Initialize status of peripherials
	for(unsigned i=0; i<KEY_CODE_COUNT; i++)
		mKeyboardStatus[i] = false;

	for(unsigned i=0; i<MAX_MOUSE_BUTTONS ; i++)
		mMouseButtonStatus[i] = false;
}

InputAdapter::~InputAdapter() {
}

} // namespace MinGLE
