//      ___                       ___           ___           ___       ___     
//     /\__\          ___        /\__\         /\  \         /\__\     /\  \.    
//    /::|  |        /\  \      /::|  |       /::\  \       /:/  /    /::\  \.   
//   /:|:|  |        \:\  \    /:|:|  |      /:/\:\  \     /:/  /    /:/\:\  \.  
//  /:/|:|__|__      /::\__\  /:/|:|  |__   /:/  \:\  \   /:/  /    /::\~\:\  \. 
// /:/ |::::\__\  __/:/\/__/ /:/ |:| /\__\ /:/__/_\:\__\ /:/__/    /:/\:\ \:\__\.
// \/__/~~/:/  / /\/:/  /    \/__|:|/:/  / \:\  /\ \/__/ \:\  \    \:\~\:\ \/__/
//       /:/  /  \::/__/         |:/:/  /   \:\ \:\__\    \:\  \    \:\ \:\__\.  
//      /:/  /    \:\__\         |::/  /     \:\/:/  /     \:\  \    \:\ \/__/  
//     /:/  /      \/__/         /:/  /       \::/  /       \:\__\    \:\__\.    
//     \/__/                     \/__/         \/__/         \/__/     \/__/    
// 
// =============================================================================
//                       Minimalist OpenGL Environment
// =============================================================================
//
// This file is part of Minimalist OpenGL Environment (MinGLE)
//
// Version: 0.2.0
// Author: Balazs Domonkos
// Filename: src/configuration/include/TextConfiguration.h
// Creation Date: January 27th 2008
// Revision Date: January 27th 2008
//
//
// The Minimalist OpenGL Environment is free software: you can
// redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// The Minimalist OpenGL Environment is distributed in the hope
// that it will be useful, but WITHOUT ANY WARRANTY; without
// even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License
// along with Minimalist OpenGL Environment.  If not, see 
// <http://www.gnu.org/licenses/>. See the COPYING file included 
// with this distribution file for license details.
//

/// @file TextConfiguration.h
/// Text configuration file support

#ifndef __MINGLE_TEXT_CONFIGURATION_H__
#define __MINGLE_TEXT_CONFIGURATION_H__

// Include class definitions
#include "classes.h"

// MinGLE headers
#include <Configuration.h>

// System headers

namespace MinGLE {

class TextConfiguration : public Configuration {

public:
	static const std::string DEFAULT_COMMENT_CHARS;
	static const std::string DEFAULT_SEPARATOR_CHARS;
	static const std::string DEFAULT_BLOCK_BEGIN_CHARS;
	static const std::string DEFAULT_BLOCK_END_CHARS;

protected:
	enum TokenType {COMMENT, SEPARATOR, BEGIN, END};

public:
	std::string mCommentChars;
	std::string mSeparatorChars;
	std::string mBeginChars;
	std::string mEndChars;

public:
			TextConfiguration(
					const std::string& filename,
					VirtualFileSystem *vfs = VirtualFileSystemManager::getDefaultVirtualFileSystem());

protected:
	void _parseTextFile(const std::string& filename, VirtualFileSystem *vfs);
	bool _findToken(std::string& line, TokenType& token,
			std::string& beforeToken);

}; // class TextConfiguration

// =============================================================================

class TextConfigurationManager : public ConfigurationManager {
public:
	static const std::string FUNCTIONALITY_STRING;

public:
	~TextConfigurationManager();
	static void registerConfigurationManager();

protected:
	virtual Configuration *_createConfiguration(const std::string& filename,
			VirtualFileSystem *vfs);
}; // class TextConfigurationManager

} // namespace MinGLE

#endif // __MINGLE_TEXT_CONFIGURATION_H__
