//      ___                       ___           ___           ___       ___     
//     /\__\          ___        /\__\         /\  \         /\__\     /\  \.    
//    /::|  |        /\  \      /::|  |       /::\  \       /:/  /    /::\  \.   
//   /:|:|  |        \:\  \    /:|:|  |      /:/\:\  \     /:/  /    /:/\:\  \.  
//  /:/|:|__|__      /::\__\  /:/|:|  |__   /:/  \:\  \   /:/  /    /::\~\:\  \. 
// /:/ |::::\__\  __/:/\/__/ /:/ |:| /\__\ /:/__/_\:\__\ /:/__/    /:/\:\ \:\__\.
// \/__/~~/:/  / /\/:/  /    \/__|:|/:/  / \:\  /\ \/__/ \:\  \    \:\~\:\ \/__/
//       /:/  /  \::/__/         |:/:/  /   \:\ \:\__\    \:\  \    \:\ \:\__\.  
//      /:/  /    \:\__\         |::/  /     \:\/:/  /     \:\  \    \:\ \/__/  
//     /:/  /      \/__/         /:/  /       \::/  /       \:\__\    \:\__\.    
//     \/__/                     \/__/         \/__/         \/__/     \/__/    
// 
// =============================================================================
//                       Minimalist OpenGL Environment
// =============================================================================
//
// This file is part of Minimalist OpenGL Environment (MinGLE)
//
// Version: 0.2.0
// Author: Balazs Domonkos
// Filename: include/Atlas.h
// Creation Date: December 12th 2007
// Revision Date: December 12th 2007
//
//
// The Minimalist OpenGL Environment is free software: you can
// redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// The Minimalist OpenGL Environment is distributed in the hope
// that it will be useful, but WITHOUT ANY WARRANTY; without
// even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License
// along with Minimalist OpenGL Environment.  If not, see 
// <http://www.gnu.org/licenses/>. See the COPYING file included 
// with this distribution file for license details.
//

/// @file LocalFileSystem.h
/// Local file system class

#ifndef __MINGLE_LOCALFILESYSTEM_H__
#define __MINGLE_LOCALFILESYSTEM_H__

// Include class definitions
#include "classes.h"

// MinGLE headers
#include "VirtualFileSystem.h"

namespace MinGLE {
class LocalFileSystem : public VirtualFileSystem {

protected:
	std::string mRootDirectory;
	
public:
	LocalFileSystem(std::string rootDirectory = "");
	virtual ~LocalFileSystem();

	virtual std::string getURL(const std::string& filename = "") const;
	
	virtual FileHandle
			open(const std::string& filename, const std::string& mode);
	virtual void close(FileHandle f);
	virtual bool eof(FileHandle f);
	virtual int getchar(FileHandle f);
	virtual int putchar(int ch, FileHandle f);
	virtual int read(void *ptr, int size, int count, FileHandle f);
	virtual int write(const void *ptr, int size, int count, FileHandle f);
	virtual int seek(FileHandle f, int numbytes, int where);
	virtual int tell(FileHandle f);

	virtual bool exists(const std::string& path);
	
	virtual DirectoryEntryList listDirectory(const std::string& directoryName,
			const unsigned mask = DIRECTORY_ENTRY_TYPE_MASK_ALL);
}; // class LocalFileSystem

// =============================================================================

class LocalFileSystemManager : public VirtualFileSystemManager {

public:
	static const std::string PROTOCOL_STRING;
	
public:
	virtual ~LocalFileSystemManager();
	static void registerVirtualFileSystemManager();

protected:
	virtual VirtualFileSystem *_createVirtualFileSystem(const std::string& connectionString);
}; // class LocalFileSystemManager

} // namespace MinGLE

#endif // __MINGLE_LOCALFILESYSTEM_H__
