//      ___                       ___           ___           ___       ___     
//     /\__\          ___        /\__\         /\  \         /\__\     /\  \.    
//    /::|  |        /\  \      /::|  |       /::\  \       /:/  /    /::\  \.   
//   /:|:|  |        \:\  \    /:|:|  |      /:/\:\  \     /:/  /    /:/\:\  \.  
//  /:/|:|__|__      /::\__\  /:/|:|  |__   /:/  \:\  \   /:/  /    /::\~\:\  \. 
// /:/ |::::\__\  __/:/\/__/ /:/ |:| /\__\ /:/__/_\:\__\ /:/__/    /:/\:\ \:\__\.
// \/__/~~/:/  / /\/:/  /    \/__|:|/:/  / \:\  /\ \/__/ \:\  \    \:\~\:\ \/__/
//       /:/  /  \::/__/         |:/:/  /   \:\ \:\__\    \:\  \    \:\ \:\__\.  
//      /:/  /    \:\__\         |::/  /     \:\/:/  /     \:\  \    \:\ \/__/  
//     /:/  /      \/__/         /:/  /       \::/  /       \:\__\    \:\__\.    
//     \/__/                     \/__/         \/__/         \/__/     \/__/    
// 
// =============================================================================
//                       Minimalist OpenGL Environment
// =============================================================================
//
// This file is part of Minimalist OpenGL Environment (MinGLE)
//
// Version: 0.2.0
// Author: Balazs Domonkos
// Filename: src/fontrendering/include/BaseFontRendering.h
// Creation Date: January 25th 2008
// Revision Date: January 25th 2008
//
//
// The Minimalist OpenGL Environment is free software: you can
// redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// The Minimalist OpenGL Environment is distributed in the hope
// that it will be useful, but WITHOUT ANY WARRANTY; without
// even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License
// along with Minimalist OpenGL Environment.  If not, see 
// <http://www.gnu.org/licenses/>. See the COPYING file included 
// with this distribution file for license details.
//

/// @file BaseFontRendering.h
/// Basic, image atlas based font rendering implementation

#ifndef __MINGLE_BASE_FONT_RENDERING_H__
#define __MINGLE_BASE_FONT_RENDERING_H__

// Include class definitions
#include <classes.h>

// MinGLE headers
#include <FontRendering.h>

namespace MinGLE {

class BaseFont : public RasterFont {

public:
	BaseFont(const std::string& filename, VirtualFileSystem *vfs,
			GLenum format = DEFAULT_FONT_FORMAT,
			GLenum type = DEFAULT_FONT_TYPE);
}; // class BaseFont

// =============================================================================

class BaseFontRendering : public FontRendering {

public:
	static const std::string IMPLEMENTATION_NAME;
	
public:
	BaseFontRendering();
	virtual ~BaseFontRendering();

protected:
	virtual Font *_createFont(const std::string& filename, VirtualFileSystem *vfs, 
			size_t height, size_t width, 
			long faceIndex, size_t firstCharacter,
			size_t lastCharacter, GLenum format, GLenum type,
			RasterFont::ImageScaleMode imageScaleMode);
	virtual void _saveFont(Font *font, const std::string& filename, VirtualFileSystem *vfs);
}; // BaseFontRendering

} // namespace MinGLE

#endif // __MINGLE_BASE_FONT_RENDERING_H__
