//      ___                       ___           ___           ___       ___     
//     /\__\          ___        /\__\         /\  \         /\__\     /\  \.    
//    /::|  |        /\  \      /::|  |       /::\  \       /:/  /    /::\  \.   
//   /:|:|  |        \:\  \    /:|:|  |      /:/\:\  \     /:/  /    /:/\:\  \.  
//  /:/|:|__|__      /::\__\  /:/|:|  |__   /:/  \:\  \   /:/  /    /::\~\:\  \. 
// /:/ |::::\__\  __/:/\/__/ /:/ |:| /\__\ /:/__/_\:\__\ /:/__/    /:/\:\ \:\__\.
// \/__/~~/:/  / /\/:/  /    \/__|:|/:/  / \:\  /\ \/__/ \:\  \    \:\~\:\ \/__/
//       /:/  /  \::/__/         |:/:/  /   \:\ \:\__\    \:\  \    \:\ \:\__\.  
//      /:/  /    \:\__\         |::/  /     \:\/:/  /     \:\  \    \:\ \/__/  
//     /:/  /      \/__/         /:/  /       \::/  /       \:\__\    \:\__\.    
//     \/__/                     \/__/         \/__/         \/__/     \/__/    
// 
// =============================================================================
//                       Minimalist OpenGL Environment
// =============================================================================
//
// This file is part of Minimalist OpenGL Environment (MinGLE)
//
// Version: 0.2.0
// Author: Balazs Domonkos
// Filename: src/imaging/include/DevilImage.h
// Creation Date: March 02th 2008
// Revision Date: March 02th 2008
//
//
// The Minimalist OpenGL Environment is free software: you can
// redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// The Minimalist OpenGL Environment is distributed in the hope
// that it will be useful, but WITHOUT ANY WARRANTY; without
// even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License
// along with Minimalist OpenGL Environment.  If not, see 
// <http://www.gnu.org/licenses/>. See the COPYING file included 
// with this distribution file for license details.
//

/// @file NativeImaging.h
/// Native imaging implementation

#ifndef __MINGLE_NATIVE_IMAGING_H__
#define __MINGLE_NATIVE_IMAGING_H__

// Include class definitions
#include <classes.h>

// MinGLE headers
#include <Imaging.h>
#include <Threading.h>

namespace MinGLE {

class NativeImage : public Image {

public:
	NativeImage();
	virtual ~NativeImage();

	// Input
	virtual void loadFromFile(const std::string& filename,
			VirtualFileSystem *vfs = VirtualFileSystemManager::getDefaultVirtualFileSystem());
	virtual void createFromMemory(unsigned width, unsigned height,
			GLenum format = DEFAULT_FORMAT, GLenum type = DEFAULT_TYPE,
			void *data = 0);
	virtual void texImage2D(GLint level = 0);

	// Output
	virtual void saveToFile(const std::string& filename,
			VirtualFileSystem *vfs = VirtualFileSystemManager::getDefaultVirtualFileSystem());
	virtual void saveToFile(const std::string& pattern, unsigned index,
			VirtualFileSystem *vfs = VirtualFileSystemManager::getDefaultVirtualFileSystem());
	virtual void saveToFile(const std::string& pattern, unsigned index1,
			unsigned index2, VirtualFileSystem *vfs = VirtualFileSystemManager::getDefaultVirtualFileSystem());

	// Convert
	virtual void convert(GLenum format, GLenum type);
};

// =============================================================================

class NativeImaging : public Imaging {

public:
	NativeImaging();
	virtual ~NativeImaging();

protected:
	virtual Image *_createImage();
	virtual Image *_createImage(unsigned width, unsigned height, GLenum format,
			GLenum type, void *data = 0);
	virtual Image *_createImage(const std::string& filename,
			VirtualFileSystem *vfs);
	virtual Image *_createImage(const Image * image);
};

} // namespace MinGLE

#endif // __MINGLE_NATIVE_IMAGING_H__
