//      ___                       ___           ___           ___       ___     
//     /\__\          ___        /\__\         /\  \         /\__\     /\  \.    
//    /::|  |        /\  \      /::|  |       /::\  \       /:/  /    /::\  \.   
//   /:|:|  |        \:\  \    /:|:|  |      /:/\:\  \     /:/  /    /:/\:\  \.  
//  /:/|:|__|__      /::\__\  /:/|:|  |__   /:/  \:\  \   /:/  /    /::\~\:\  \. 
// /:/ |::::\__\  __/:/\/__/ /:/ |:| /\__\ /:/__/_\:\__\ /:/__/    /:/\:\ \:\__\.
// \/__/~~/:/  / /\/:/  /    \/__|:|/:/  / \:\  /\ \/__/ \:\  \    \:\~\:\ \/__/
//       /:/  /  \::/__/         |:/:/  /   \:\ \:\__\    \:\  \    \:\ \:\__\.  
//      /:/  /    \:\__\         |::/  /     \:\/:/  /     \:\  \    \:\ \/__/  
//     /:/  /      \/__/         /:/  /       \::/  /       \:\__\    \:\__\.    
//     \/__/                     \/__/         \/__/         \/__/     \/__/    
// 
// =============================================================================
//                       Minimalist OpenGL Environment
// =============================================================================
//
// This file is part of Minimalist OpenGL Environment (MinGLE)
//
// Version: 0.2.0
// Author: Balazs Domonkos
// Filename: src/imaging/src/DevilImage.cpp
// Creation Date: March 02th 2008
// Revision Date: March 02th 2008
//
//
// The Minimalist OpenGL Environment is free software: you can
// redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// The Minimalist OpenGL Environment is distributed in the hope
// that it will be useful, but WITHOUT ANY WARRANTY; without
// even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License
// along with Minimalist OpenGL Environment.  If not, see 
// <http://www.gnu.org/licenses/>. See the COPYING file included 
// with this distribution file for license details.
//

/// @file NativeImaging.cpp
/// Native imaging implementation

// Header that contains the declaration
#include <NativeImaging.h>

// MinGLE headers
#include <Exception.h>
#include <System.h>

namespace MinGLE {

// Dynamic library loader function
extern "C"void loadDynLib(void) {
	// Create and register DevIL imaging system implementation
	Imaging::registerImplementation("native", new NativeImaging());
}

// Dynamic library unloader function
extern "C" void unloadDynLib(void) {
	// Nothing to do
}

// =============================================================================

NativeImage::NativeImage() : Image() {
}

NativeImage::~NativeImage() {
}

void NativeImage::loadFromFile(const std::string& filename, VirtualFileSystem *vfs) {
}

void NativeImage::createFromMemory(unsigned width, unsigned height,
		GLenum format, GLenum type, void *data) {
}

void NativeImage::texImage2D(GLint level) {
}

void NativeImage::saveToFile(const std::string& filename, VirtualFileSystem *vfs) {
}

void NativeImage::saveToFile(const std::string& pattern, unsigned index, VirtualFileSystem *vfs) {
}

void NativeImage::saveToFile(const std::string& pattern, unsigned index1, unsigned index2, VirtualFileSystem *vfs) {
}

void NativeImage::convert(GLenum format, GLenum type) {
	
}

// =============================================================================

NativeImaging::NativeImaging() {
}

NativeImaging::~NativeImaging() {
	// Nothing to do
}

Image *NativeImaging::_createImage() {
	return new NativeImage();
}

Image *NativeImaging::_createImage(unsigned width, unsigned height, GLenum format,
		GLenum type, void *data) {
	//return new NativeImage(width, height, format, type, data);
	return 0;
}

Image *NativeImaging::_createImage(const std::string& filename, VirtualFileSystem *vfs) {
	//return new NativeImage(filename, vfs);
	return 0;
}

Image *NativeImaging::_createImage(const Image * image) {
	//return new NativeImage(image);
	return 0;
}

} // namespace MinGLE
