//      ___                       ___           ___           ___       ___     
//     /\__\          ___        /\__\         /\  \         /\__\     /\  \.    
//    /::|  |        /\  \      /::|  |       /::\  \       /:/  /    /::\  \.   
//   /:|:|  |        \:\  \    /:|:|  |      /:/\:\  \     /:/  /    /:/\:\  \.  
//  /:/|:|__|__      /::\__\  /:/|:|  |__   /:/  \:\  \   /:/  /    /::\~\:\  \. 
// /:/ |::::\__\  __/:/\/__/ /:/ |:| /\__\ /:/__/_\:\__\ /:/__/    /:/\:\ \:\__\.
// \/__/~~/:/  / /\/:/  /    \/__|:|/:/  / \:\  /\ \/__/ \:\  \    \:\~\:\ \/__/
//       /:/  /  \::/__/         |:/:/  /   \:\ \:\__\    \:\  \    \:\ \:\__\.  
//      /:/  /    \:\__\         |::/  /     \:\/:/  /     \:\  \    \:\ \/__/  
//     /:/  /      \/__/         /:/  /       \::/  /       \:\__\    \:\__\.    
//     \/__/                     \/__/         \/__/         \/__/     \/__/    
// 
// =============================================================================
//                       Minimalist OpenGL Environment
// =============================================================================
//
// This file is part of Minimalist OpenGL Environment (MinGLE)
//
// Version: 0.2.0
// Author: Balazs Domonkos
// Filename: src/GLUT/include/GLUTSystem.h
// Creation Date: December 12th 2007
// Revision Date: December 12th 2007
//
//
// The Minimalist OpenGL Environment is free software: you can
// redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// The Minimalist OpenGL Environment is distributed in the hope
// that it will be useful, but WITHOUT ANY WARRANTY; without
// even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License
// along with Minimalist OpenGL Environment.  If not, see 
// <http://www.gnu.org/licenses/>. See the COPYING file included 
// with this distribution file for license details.
//

/// @file GLUTSystem.h
/// GLUT system manager class

#ifndef __MINGLE_GLUTSYSTEM_H__
#define __MINGLE_GLUTSYSTEM_H__

// Include class definitions
#include <classes.h>

// MinGLE headers
#include <System.h>
#include <GLUTWindow.h>

namespace MinGLE {

class GLUTSystem : public System {

protected:
	/// The GLUT system can only have one window
	GLUTWindow *mWindow;

protected:
	virtual bool _initialize(int *argc, char **argv);

	virtual Window
			* _createWindow(
					unsigned width = Window::DEFAULT_WIDTH,
					unsigned height = Window::DEFAULT_HEIGHT,
					WindowAttributes::ColorMode colorMode = WindowAttributes::DIRECT_COLORS,
					bool doubleBuffer = true, bool fullScreen = false,
					unsigned videoRate = 0, unsigned char *colorBits = 0,
					unsigned char depthBits = 16,
					unsigned char stencilBits = 0,
					unsigned char *accumColorBits = 0, unsigned auxBuffers = 0,
					bool stereo = false);

	virtual void _enterMainLoop();
	virtual void _quit();

	virtual double _getTime();

public:
	GLUTSystem();
	virtual ~GLUTSystem();

}; // class GLUTSystem

} // namespace MinGLE

#endif // __MINGLE_SYSTEM_H__
