//      ___                       ___           ___           ___       ___     
//     /\__\          ___        /\__\         /\  \         /\__\     /\  \.    
//    /::|  |        /\  \      /::|  |       /::\  \       /:/  /    /::\  \.   
//   /:|:|  |        \:\  \    /:|:|  |      /:/\:\  \     /:/  /    /:/\:\  \.  
//  /:/|:|__|__      /::\__\  /:/|:|  |__   /:/  \:\  \   /:/  /    /::\~\:\  \. 
// /:/ |::::\__\  __/:/\/__/ /:/ |:| /\__\ /:/__/_\:\__\ /:/__/    /:/\:\ \:\__\.
// \/__/~~/:/  / /\/:/  /    \/__|:|/:/  / \:\  /\ \/__/ \:\  \    \:\~\:\ \/__/
//       /:/  /  \::/__/         |:/:/  /   \:\ \:\__\    \:\  \    \:\ \:\__\.  
//      /:/  /    \:\__\         |::/  /     \:\/:/  /     \:\  \    \:\ \/__/  
//     /:/  /      \/__/         /:/  /       \::/  /       \:\__\    \:\__\.    
//     \/__/                     \/__/         \/__/         \/__/     \/__/    
// 
// =============================================================================
//                       Minimalist OpenGL Environment
// =============================================================================
//
// This file is part of Minimalist OpenGL Environment (MinGLE)
//
// Version: 0.2.0
// Author: Balazs Domonkos
// Filename: src/threading/include/PthThread.h
// Creation Date: December 13th 2007
// Revision Date: December 13th 2007
//
//
// The Minimalist OpenGL Environment is free software: you can
// redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// The Minimalist OpenGL Environment is distributed in the hope
// that it will be useful, but WITHOUT ANY WARRANTY; without
// even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License
// along with Minimalist OpenGL Environment.  If not, see 
// <http://www.gnu.org/licenses/>. See the COPYING file included 
// with this distribution file for license details.
//

/// @file PthThread.h
/// PTH threading implementation

#ifndef __MINGLE_PTH_THREADING_H__
#define __MINGLE_PTH_THREADING_H__

// Include class definitions
#include <classes.h>

// MinGLE headers
#include <Threading.h>

// System headers
#define PTH_SYSCALL_SOFT 0
#include <pth.h>

namespace MinGLE {

class PthMutex : public Mutex {
protected:
	pth_mutex_t mMutex;

public:
	PthMutex();
	~PthMutex();

	virtual void lock();
	virtual void release();

}; // class PthMutex

// =============================================================================

class PthThread : public Thread {

protected:
	pth_t mThread;

public:
	PthThread(Runnable *runnable, bool joinable = DEFAULT_JOINABLE_FLAG);

protected:
	virtual void _startThread();
	virtual void _joinThread();
	virtual void _killThread();
	
private:
	static void *_entryPoint(void *arg);
	
}; // class PthThread

// =============================================================================

class PthThreading : public Threading {

public:
	PthThreading();
	~PthThreading();

protected:
	virtual Mutex* _createMutex();
	virtual Thread* _createThread(Runnable *runnable, bool joinable);
};

} // namespace MinGLE

#endif // __MINGLE_PTH_THREADING_H__
