// 
//    Distributed SPH Fluid Simulator (asterix@index.hu)
// =============================================================================
//
// Copyright 2007 David Illes
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 

#include "../include/particle.hpp"

#include <stdio.h>
#include <math.h>

//-----------------------------------------------------------------------------
// initialize the particle
//-----------------------------------------------------------------------------
void Particle::init(int i,int li,float m,float x,float y,float z, float velx, float vely, float velz, bool rcv, bool mnull, bool mthrow) {
//-----------------------------------------------------------------------------
	// set particle id
	if(i!=-1) id = i;
	cellid = -1;
	
	// per node local ide
	lid = li;
	
	// mass
	mass = m;
	
	// particle attributes
	position.Set(x,y,z);
	velocity.Set(velx,vely,velz);
	density = 0;
	pressure = 0;
	defposition.Set(x,y,z);
	cellx = celly = cellz = -1;
	
	// particle exists, received, to be thrown
	isNull = mnull;
	received = rcv;
	toThrow = mthrow;

	// number of neighbouring particles
	num_of_neighbours = 0;
}

//-----------------------------------------------------------------------------
// reset position
//-----------------------------------------------------------------------------
void Particle::resetPosition() { position.Set(defposition); }
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// reset status
//-----------------------------------------------------------------------------
void Particle::resetStatus() { toThrow = false; received=false; }
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// add a neighbour to this particle
//-----------------------------------------------------------------------------
void Particle::newNeighbour(Particle* p) {
//-----------------------------------------------------------------------------
	neighbours[num_of_neighbours++].init(p);
}

//-----------------------------------------------------------------------------
void Particle::resetNeighbours() {
//-----------------------------------------------------------------------------
	num_of_neighbours = 0;
}
