// 
//    Distributed SPH Fluid Simulator (asterix@index.hu)
// =============================================================================
//
// Copyright 2007 David Illes
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 

#include "../include/scene.hpp"
#include "../include/objectreader.hpp"

//----------------------------------------------------------------------
// init scene
//----------------------------------------------------------------------
Scene::Scene() {
//----------------------------------------------------------------------
	num_of_particlesystems = num_of_maincontainers = num_of_containers = num_of_cameras = 0;
}

//----------------------------------------------------------------------
// read objects from a file
//----------------------------------------------------------------------
void Scene::initScene(const char* file) {
//----------------------------------------------------------------------
	ObjectReader* reader = new ObjectReader(file);
	bool firstcam = true;
	bool firstps = true;
	bool firstcon = true;
	
	// create cameras
	Camera* cam;
	while (cam=reader->nextCamera()) {
		num_of_cameras++;
		cameras.push_back(cam);
		// set the active camera
		if (firstcam) {
			active_camera = cam;
			firstcam = false;
		}
	}
	// default camera - when no camera description in file
	if (firstcam) {
		cam = new Camera();
		cameras.push_back(cam);
		active_camera = cam;
	}

	// create particlesystems
	ParticleSystem* ps;
	while (ps=reader->nextPartSystem()) {
		num_of_particlesystems++;
		particlesystems.push_back(ps);
		if (firstps) {
			firstps = false;
			active_particlesystem = ps;
		}
	}

	// create containers
	Container* con;
	while (con=reader->nextContainer()) {
		con->distribute();
		con->initSystem(particlesystems);
		// create grid
		con->createGrid();
		// add to maincontainers
		num_of_maincontainers++;
		maincontainers.push_back(con);
		// get containers number
		con->getSubNumber(&num_of_containers);

		if (firstcon && con->partsysid >= 0) {
			active_container = con;
			firstcon = false;
		}
	}
	// collect all containers
	containers = new Container*[num_of_containers];
	for(unsigned int i=0;i<maincontainers.size();i++) {
		maincontainers[i]->getSubs(containers);
	}	
		
}

//----------------------------------------------------------------------
// move the scene
//----------------------------------------------------------------------
void Scene::newFrame() {
//----------------------------------------------------------------------
	for(int i=0;i<num_of_containers;i++)
		containers[i]->move(containers,num_of_containers);
}

//----------------------------------------------------------------------
// draw the scene
//----------------------------------------------------------------------
void Scene::display() {
//----------------------------------------------------------------------
	// draw containers - with the fluid within
	for(int i=0;i<num_of_containers;i++)
		containers[i]->draw();
}

//----------------------------------------------------------------------
// switch camera
//----------------------------------------------------------------------
void Scene::switchCamera() {
//----------------------------------------------------------------------
	if (active_camera->getID() == (unsigned int)cameras.size()-1) active_camera = cameras[0];
	else active_camera = cameras[(active_camera->getID()+1)];
}

//----------------------------------------------------------------------
// switch particlesystem
//----------------------------------------------------------------------
void Scene::switchParticleSystem() {
//----------------------------------------------------------------------
	if (active_particlesystem->getID() == particlesystems.size()-1)
		active_particlesystem = particlesystems[0];
	else active_particlesystem = particlesystems[(active_particlesystem->getID()+1)];
}
